@@ -2,7 +2,7 @@
 #
 # Classes for the Hatari UI dialogs
 #
-# Copyright (C) 2008-2011 by Eero Tamminen <eerot at berlios>
+# Copyright (C) 2008-2012 by Eero Tamminen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -300,7 +300,7 @@ class FloppyDialog(HatariUIDialog):
         row += 1
         fastfdc = gtk.CheckButton("Fast floppy access")
         fastfdc.set_active(config.get_fastfdc())
-        fastfdc.set_tooltip_text("Can cause incompatibilites with some games/demos")
+        fastfdc.set_tooltip_text("Can cause incompatibilities with some games/demos")
         table_add_widget_row(table, row, None, fastfdc)
 
         table.show_all()
@@ -422,8 +422,9 @@ class DisplayDialog(HatariUIDialog):
         skip.set_tooltip_text("Set how many frames are skipped")
 
         maxw, maxh = config.get_max_size()
-        maxadjw = gtk.Adjustment(maxw, 320, 1280, 8, 40)
-        maxadjh = gtk.Adjustment(maxh, 200,  960, 8, 40)
+        topw, toph = config.get_desktop_size()
+        maxadjw = gtk.Adjustment(maxw, 320, topw, 8, 40)
+        maxadjh = gtk.Adjustment(maxh, 200, toph, 8, 40)
         scalew = gtk.HScale(maxadjw)
         scaleh = gtk.HScale(maxadjh)
         scalew.set_digits(0)
@@ -431,13 +432,21 @@ class DisplayDialog(HatariUIDialog):
         scalew.set_tooltip_text("Preferred/maximum zoomed width")
         scaleh.set_tooltip_text("Preferred/maximum zoomed height")
 
-        desktop = gtk.CheckButton("Keep desktop resolution")
-        desktop.set_active(config.get_desktop())
-        desktop.set_tooltip_text("Whether to keep desktop resolution in fullscreen and (try to) scale Atari screen by an integer factor instead")
+        force_max = gtk.CheckButton("Force max resolution (Falcon)")
+        force_max.set_active(config.get_force_max())
+        force_max.set_tooltip_text("Whether to force maximum resolution to help recording videos of demos which do resolution changes")
 
-        borders = gtk.CheckButton("ST/STE overscan borders")
+        desktop = gtk.CheckButton("Keep desktop resolution (Falcon/TT)")
+        desktop.set_active(config.get_desktop())
+        desktop.set_tooltip_text("Whether to keep screen resolution in fullscreen and (try to) scale Atari screen by an integer factor instead")
+
+        desktop_st = gtk.CheckButton("Keep desktop resolution (ST/STE)")
+        desktop_st.set_active(config.get_desktop_st())
+        desktop_st.set_tooltip_text("Whether to keep screen resolution in fullscreen to avoid potential sound skips + delay (NO SCALING)")
+
+        borders = gtk.CheckButton("Atari screen borders")
         borders.set_active(config.get_borders())
-        borders.set_tooltip_text("Whether to show overscan borders in ST/STE low/mid-rez. Visible border area is affected by max. zoom size")
+        borders.set_tooltip_text("Whether to show overscan borders in ST/STE low/mid-rez or in Falcon color resolutions. Visible border area is affected by max. zoom size")
 
         statusbar = gtk.CheckButton("Show statusbar")
         statusbar.set_active(config.get_statusbar())
@@ -461,7 +470,9 @@ class DisplayDialog(HatariUIDialog):
         dialog.vbox.add(gtk.Label("Max zoomed size:"))
         dialog.vbox.add(scalew)
         dialog.vbox.add(scaleh)
+        dialog.vbox.add(force_max)
         dialog.vbox.add(desktop)
+        dialog.vbox.add(desktop_st)
         dialog.vbox.add(borders)
         dialog.vbox.add(statusbar)
         dialog.vbox.add(led)
@@ -472,7 +483,9 @@ class DisplayDialog(HatariUIDialog):
         self.skip = skip
         self.maxw = maxadjw
         self.maxh = maxadjh
+        self.force_max = force_max
         self.desktop = desktop
+        self.desktop_st = desktop_st
         self.borders = borders
         self.statusbar = statusbar
         self.led = led
@@ -488,7 +501,9 @@ class DisplayDialog(HatariUIDialog):
             config.lock_updates()
             config.set_frameskip(self.skip.get_active())
             config.set_max_size(self.maxw.get_value(), self.maxh.get_value())
+            config.set_force_max(self.force_max.get_active())
             config.set_desktop(self.desktop.get_active())
+            config.set_desktop_st(self.desktop_st.get_active())
             config.set_borders(self.borders.get_active())
             config.set_statusbar(self.statusbar.get_active())
             config.set_led(self.led.get_active())
@@ -620,24 +635,36 @@ class PathDialog(HatariUIDialog):
 class SoundDialog(HatariUIDialog):
 
     def _create_dialog(self, config):
-        combo = gtk.combo_box_entry_new_text()
-        for text in config.get_sound_values():
-            combo.append_text(text)
-        enabled, hz = config.get_sound()
+        enabled, curhz = config.get_sound()
+
         self.enabled = gtk.CheckButton("Sound enabled")
         self.enabled.set_active(enabled)
-        combo.child.set_text(hz)
-        box = gtk.HBox()
-        box.pack_start(gtk.Label("Sound frequency:"), False, False)
-        box.add(combo)
-        self.sound = combo.child
+
+        hz = gtk.combo_box_new_text()
+        for text in config.get_sound_values():
+            hz.append_text(text)
+        hz.set_active(curhz)
+        self.hz = hz
+
+        ymmixer = gtk.combo_box_new_text()
+        for text in config.get_ymmixer_types():
+            ymmixer.append_text(text)
+        ymmixer.set_active(config.get_ymmixer())
+        self.ymmixer = ymmixer
+
+        self.mic = gtk.CheckButton("Enable (Falcon) microphone")
+        self.mic.set_active(config.get_mic())
 
         dialog = gtk.Dialog("Sound settings", self.parent,
             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
             (gtk.STOCK_APPLY,  gtk.RESPONSE_APPLY,
              gtk.STOCK_CANCEL,  gtk.RESPONSE_CANCEL))
         dialog.vbox.add(self.enabled)
-        dialog.vbox.add(box)
+        dialog.vbox.add(gtk.Label("Sound frequency::"))
+        dialog.vbox.add(hz)
+        dialog.vbox.add(gtk.Label("YM voices mixing method:"))
+        dialog.vbox.add(ymmixer)
+        dialog.vbox.add(self.mic)
         dialog.vbox.show_all()
         self.dialog = dialog
 
@@ -648,11 +675,13 @@ class SoundDialog(HatariUIDialog):
         response = self.dialog.run()
         self.dialog.hide()
         if response == gtk.RESPONSE_APPLY:
+            config.lock_updates()
+            config.set_mic(self.mic.get_active())
+            config.set_ymmixer(self.ymmixer.get_active())
             enabled = self.enabled.get_active()
-            hz1 = self.sound.get_text()
-            hz2 = config.set_sound(enabled, hz1)
-            if hz2 != hz1:
-                self.sound.set_text(hz2)
+            hz = self.hz.get_active()
+            config.set_sound(enabled, hz)
+            config.flush_updates()
         
 
 # ---------------------------
@@ -704,7 +733,7 @@ class TraceDialog(HatariUIDialog):
         "dsp_state"
     ]
     def __init__(self, parent):
-        self.savedpoints = "none"
+        self.savedpoints = None
         hbox1 = gtk.HBox()
         hbox1.add(create_button("Load", self._load_traces))
         hbox1.add(create_button("Clear", self._clear_traces))
@@ -746,6 +775,8 @@ class TraceDialog(HatariUIDialog):
 
     def _set_traces(self, tracepoints):
         self._clear_traces()
+        if not tracepoints:
+            return
         for trace in tracepoints.split(","):
             if trace in self.tracewidgets:
                 self.tracewidgets[trace].set_active(True)
