@@ -114,6 +114,10 @@ bool Change_DoNeedReset(CNF_PARAMS *current, CNF_PARAMS *changed)
 #endif
 
 #if ENABLE_WINUAE_CPU
+	/* Did change CPU address mode? */
+	if (changed->System.bAddressSpace24 != current->System.bAddressSpace24)
+		return true;
+
 	/* Did change CPU prefetch mode? */
 	if (changed->System.bCompatibleCpu != current->System.bCompatibleCpu)
 		return true;
@@ -125,6 +129,10 @@ bool Change_DoNeedReset(CNF_PARAMS *current, CNF_PARAMS *changed)
 	/* Did change MMU? */
 	if (changed->System.bMMU != current->System.bMMU)
 		return true;
+ 
+	/* Did change FPU? */
+	if (changed->System.n_FPUType != current->System.n_FPUType)
+		return true;
 #endif
 
 	/* Did change size of memory? */
@@ -148,6 +156,7 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 	bool bReInitIoMem = false;
 	bool bScreenModeChange = false;
 	bool bReInitMidi = false;
+	bool bReInitPrinter = false;
 	bool bFloppyInsert[MAX_FLOPPYDRIVES];
 	int i;
 
@@ -175,11 +184,11 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 
 	/* Did set new printer parameters? */
 	if (changed->Printer.bEnablePrinting != current->Printer.bEnablePrinting
-	    || changed->Printer.bPrintToFile != current->Printer.bPrintToFile
 	    || strcmp(changed->Printer.szPrintToFileName,current->Printer.szPrintToFileName))
 	{
 		Dprintf("- printer>\n");
-		Printer_CloseAllConnections();
+		Printer_UnInit();
+		bReInitPrinter = true;
 	}
 
 	/* Did set new RS232 parameters? */
@@ -357,7 +366,7 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 	}
 
 	/* Re-initialize the RS232 emulation: */
-	if (ConfigureParams.RS232.bEnableRS232 && !bConnectedRS232)
+	if (ConfigureParams.RS232.bEnableRS232)
 	{
 		Dprintf("- RS-232<\n");
 		RS232_Init();
@@ -370,6 +379,13 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 		IoMem_Init();
 	}
 
+	/* Re-init Printer emulation? */
+	if (bReInitPrinter)
+	{
+		Dprintf("- printer<\n");
+		Printer_Init();
+	}
+
 	/* Re-init MIDI emulation? */
 	if (bReInitMidi)
 	{
@@ -442,7 +458,8 @@ static bool Change_Options(int argc, const char *argv[])
 
 /*-----------------------------------------------------------------------*/
 /**
- * Parse given command line and change Hatari options accordingly
+ * Parse given command line and change Hatari options accordingly.
+ * Given string must be stripped and not empty.
  * Return false if parsing failed or there were no args, true otherwise
  */
 bool Change_ApplyCommandline(char *cmdline)
@@ -455,7 +472,7 @@ bool Change_ApplyCommandline(char *cmdline)
 	inarg = argc = 0;
 	for (i = 0; cmdline[i]; i++)
 	{
-		if (isspace(cmdline[i]))
+		if (isspace(cmdline[i]) && cmdline[i-1] != '\\')
 		{
 			inarg = 0;
 			continue;
@@ -486,13 +503,22 @@ bool Change_ApplyCommandline(char *cmdline)
 	{
 		if (isspace(cmdline[i]))
 		{
-			cmdline[i] = '\0';
-			if (inarg)
+			if (cmdline[i-1] != '\\')
 			{
-				fprintf(stderr, "- '%s'\n", argv[argc-1]);
+				cmdline[i] = '\0';
+				if (inarg)
+				{
+					fprintf(stderr, "- '%s'\n", argv[argc-1]);
+				}
+				inarg = 0;
+				continue;
+			}
+			else
+			{
+				/* remove quote for space */
+				memcpy(cmdline+i-1, cmdline+i, strlen(cmdline+i)+1);
+				i--;
 			}
-			inarg = 0;
-			continue;
 		}
 		if (!inarg)
 		{
