@@ -232,6 +232,7 @@ static const struct Config_Tag configs_Sound[] =
 {
 	{ "bEnableMicrophone", Bool_Tag, &ConfigureParams.Sound.bEnableMicrophone },
 	{ "bEnableSound", Bool_Tag, &ConfigureParams.Sound.bEnableSound },
+	{ "bEnableSoundSync", Bool_Tag, &ConfigureParams.Sound.bEnableSoundSync },
 	{ "nPlaybackFreq", Int_Tag, &ConfigureParams.Sound.nPlaybackFreq },
 	{ "nSdlAudioBufferSize", Int_Tag, &ConfigureParams.Sound.SdlAudioBufferSize },
 	{ "szYMCaptureFileName", String_Tag, ConfigureParams.Sound.szYMCaptureFileName },
@@ -302,7 +303,6 @@ static const struct Config_Tag configs_Rs232[] =
 static const struct Config_Tag configs_Printer[] =
 {
 	{ "bEnablePrinting", Bool_Tag, &ConfigureParams.Printer.bEnablePrinting },
-	{ "bPrintToFile", Bool_Tag, &ConfigureParams.Printer.bPrintToFile },
 	{ "szPrintToFileName", String_Tag, ConfigureParams.Printer.szPrintToFileName },
 	{ NULL , Error_Tag, NULL }
 };
@@ -462,7 +462,6 @@ void Configuration_SetDefault(void)
 
 	/* Set defaults for Printer */
 	ConfigureParams.Printer.bEnablePrinting = false;
-	ConfigureParams.Printer.bPrintToFile = true;
 	sprintf(ConfigureParams.Printer.szPrintToFileName, "%s%chatari.prn",
 	        psHomeDir, PATHSEP);
 
@@ -493,14 +492,15 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Screen.bShowStatusbar = true;
 	ConfigureParams.Screen.bShowDriveLed = true;
 	ConfigureParams.Screen.bCrop = false;
-	/* default to desktop size */
+	/* gives zoomed Falcon/TT windows about same size as ST/STE windows */
+	ConfigureParams.Screen.nMaxWidth = 2*(48+320+48);
+	ConfigureParams.Screen.nMaxHeight = 2*NUM_VISIBLE_LINES+24;
 	ConfigureParams.Screen.bForceMax = false;
-	ConfigureParams.Screen.nMaxWidth = 0;
-	ConfigureParams.Screen.nMaxHeight = 0;
 
 	/* Set defaults for Sound */
 	ConfigureParams.Sound.bEnableMicrophone = true;
 	ConfigureParams.Sound.bEnableSound = true;
+	ConfigureParams.Sound.bEnableSoundSync = false;
 	ConfigureParams.Sound.nPlaybackFreq = 44100;
 	sprintf(ConfigureParams.Sound.szYMCaptureFileName, "%s%chatari.wav",
 	        psWorkingDir, PATHSEP);
@@ -514,23 +514,30 @@ void Configuration_SetDefault(void)
 	strcpy(ConfigureParams.Rom.szCartridgeImageFileName, "");
 
 	/* Set defaults for System */
-	ConfigureParams.System.nCpuLevel = 0;
-	ConfigureParams.System.nCpuFreq = 8;
-	ConfigureParams.System.bCompatibleCpu = true;
+#if ENABLE_WINUAE_CPU
+	/* Default to Falcon with WinUAE CPU core... */
+	ConfigureParams.System.nMachineType = MACHINE_FALCON;
+	ConfigureParams.System.nCpuLevel = 3;
+	ConfigureParams.System.nCpuFreq = 16;
+	ConfigureParams.System.nDSPType = DSP_TYPE_EMU;
+	ConfigureParams.System.bAddressSpace24 = true;
+	ConfigureParams.System.n_FPUType = FPU_NONE;
+	ConfigureParams.System.bCompatibleFPU = true;
+	ConfigureParams.System.bMMU = false;
+	ConfigureParams.System.bCycleExactCpu = true;
+#else
+	/* ...and to ST with old UAE CPU core */
 	ConfigureParams.System.nMachineType = MACHINE_ST;
-	ConfigureParams.System.bBlitter = false;
+ 	ConfigureParams.System.nCpuLevel = 0;
+ 	ConfigureParams.System.nCpuFreq = 8;
 	ConfigureParams.System.nDSPType = DSP_TYPE_NONE;
+#endif
+	ConfigureParams.System.bCompatibleCpu = true;
+	ConfigureParams.System.bBlitter = false;
 	ConfigureParams.System.bPatchTimerD = true;
 	ConfigureParams.System.bFastBoot = true;
 	ConfigureParams.System.bRealTimeClock = true;
 	ConfigureParams.System.bFastForward = false;
-#if ENABLE_WINUAE_CPU
-	ConfigureParams.System.bAddressSpace24 = true;
-	ConfigureParams.System.bCycleExactCpu = false;
-	ConfigureParams.System.n_FPUType = FPU_NONE;
-	ConfigureParams.System.bCompatibleFPU = false;
-	ConfigureParams.System.bMMU = false;
-#endif
 
 	/* Set defaults for Video */
 #if HAVE_LIBPNG
