@@ -83,10 +83,11 @@ static void (* const pIntHandlerFunctions[MAX_INTERRUPTS])(void) =
 	MFP_InterruptHandler_TimerC,
 	MFP_InterruptHandler_TimerD,
 	IKBD_InterruptHandler_ResetTimer,
-	IKBD_InterruptHandler_ACIA,
+	IKBD_InterruptHandler_ACIA_TX,
+	IKBD_InterruptHandler_ACIA_RX,
 	IKBD_InterruptHandler_MFP,
 	IKBD_InterruptHandler_AutoSend,
-	DmaSnd_InterruptHandler_Microwire,
+	DmaSnd_InterruptHandler_Microwire, /* Used for both STE and Falcon Microwire emulation */
 	Crossbar_InterruptHandler_25Mhz,
 	Crossbar_InterruptHandler_32Mhz,
 	FDC_InterruptHandler_Update,
