@@ -13,6 +13,9 @@ extern bool BreakCond_Save(const char *filename);
 extern char *BreakCond_MatchCpuVariable(const char *text, int state);
 extern char *BreakCond_MatchDspVariable(const char *text, int state);
 
+/* for evaluate.c */
+extern bool BreakCond_GetHatariVariable(const char *name, Uint32 *value);
+
 /* for debugcpu.c & debugdsp.c */
 extern const char BreakCond_Description[];
 extern const char BreakAddr_Description[];
