@@ -12,6 +12,7 @@ const char DebugInfo_fileid[] = "Hatari debuginfo.c : " __DATE__ " " __TIME__;
 #include <stdio.h>
 #include <assert.h>
 #include "main.h"
+#include "bios.h"
 #include "configuration.h"
 #include "debugInfo.h"
 #include "debugcpu.h"
@@ -21,6 +22,7 @@ const char DebugInfo_fileid[] = "Hatari debuginfo.c : " __DATE__ " " __TIME__;
 #include "evaluate.h"
 #include "file.h"
 #include "gemdos.h"
+#include "history.h"
 #include "ioMem.h"
 #include "m68000.h"
 #include "stMemory.h"
@@ -28,6 +30,7 @@ const char DebugInfo_fileid[] = "Hatari debuginfo.c : " __DATE__ " " __TIME__;
 #include "screen.h"
 #include "vdi.h"
 #include "video.h"
+#include "xbios.h"
 
 
 /* ------------------------------------------------------------------
@@ -808,6 +811,7 @@ static const struct {
 } infotable[] = {
 	{ false,"aes",       AES_Info,             NULL, "Show AES vector contents (with <value>, show opcodes)" },
 	{ false,"basepage",  DebugInfo_Basepage,   NULL, "Show program basepage info at given <address>" },
+	{ false,"bios",      Bios_Info,            NULL, "Show BIOS opcodes" },
 	{ false,"cookiejar", DebugInfo_Cookiejar,  NULL, "Show TOS Cookiejar contents" },
 	{ false,"crossbar",  DebugInfo_Crossbar,   NULL, "Show Falcon crossbar HW register values" },
 	{ true, "default",   DebugInfo_Default,    NULL, "Show default debugger entry information" },
@@ -819,16 +823,18 @@ static const struct {
 #endif
 	{ true, "file",      DebugInfo_FileParse, DebugInfo_FileArgs, "Parse commands from given debugger input <file>" },
 	{ false,"gemdos",    GemDOS_Info,          NULL, "Show GEMDOS HDD emu info (with <value>, show opcodes)" },
+	{ true, "history",   History_Show,         NULL, "Show history of last <count> instructions" },
 	{ true, "memdump",   DebugInfo_CpuMemDump, NULL, "Dump CPU memory from given <address>" },
 	{ false,"osheader",  DebugInfo_OSHeader,   NULL, "Show TOS OS header information" },
 	{ true, "regaddr",   DebugInfo_RegAddr, DebugInfo_RegAddrArgs, "Show <disasm|memdump> from CPU/DSP address pointed by <register>" },
 	{ true, "registers", DebugInfo_CpuRegister,NULL, "Show CPU registers values" },
 	{ false,"vdi",       VDI_Info,             NULL, "Show VDI vector contents (with <value>, show opcodes)" },
 	{ false,"videl",     DebugInfo_Videl,      NULL, "Show Falcon Videl HW registers values" },
-	{ false,"video",     DebugInfo_Video,      NULL, "Show Video related values" }
+	{ false,"video",     DebugInfo_Video,      NULL, "Show Video related values" },
+	{ false,"xbios",     XBios_Info,           NULL, "Show XBIOS opcodes" }
 };
 
-static int LockedFunction = 4; /* index for the "default" function */
+static int LockedFunction = 5; /* index for the "default" function */
 static Uint32 LockedArgument;
 
 /**
