@@ -10,6 +10,12 @@
 #ifndef HATARI_DEBUGINFO_H
 #define HATARI_DEBUGINFO_H
 
+/* for breakcond.c */
+extern Uint32 DebugInfo_GetTEXT(void);
+extern Uint32 DebugInfo_GetDATA(void);
+extern Uint32 DebugInfo_GetBSS(void);
+
+/* for debugui.c */
 extern void DebugInfo_ShowSessionInfo(void);
 extern char *DebugInfo_MatchInfo(const char *text, int state);
 extern char *DebugInfo_MatchLock(const char *text, int state);
