@@ -26,16 +26,13 @@ extern FILE *debugOutput;
 
 extern void DebugUI_PrintCmdHelp(const char *psCmd);
 
-extern void DebugCpu_SetDebugging(void);
 extern int DebugCpu_Init(const dbgcommand_t **table);
 extern void DebugCpu_InitSession(void);
 
 #ifdef ENABLE_DSP_EMU
-extern void DebugDsp_SetDebugging(void);
 extern int DebugDsp_Init(const dbgcommand_t **table);
 extern void DebugDsp_InitSession(void);
 #else /* !ENABLE_DSP_EMU */
-#define DebugDsp_SetDebugging()
 static inline int DebugDsp_Init(const dbgcommand_t **t) { t = NULL; return 0; }
 #define DebugDsp_InitSession()
 #endif /* !ENABLE_DSP_EMU */
