@@ -139,24 +139,9 @@ static int DebugCpu_SaveBin(int nArgc, char *psArgs[])
  */
 static void DebugCpu_ShowAddressInfo(Uint32 addr)
 {
-	Uint32 count, cycles;
-	const char *symbol;
-	bool shown = false;
-
-	symbol = Symbols_GetByCpuAddress(addr);
+	const char *symbol = Symbols_GetByCpuAddress(addr);
 	if (symbol)
-	{
-		fprintf(debugOutput, "%s", symbol);
-		shown = true;
-	}
-	if (Profile_CpuAddressData(addr, &count, &cycles))
-	{
-		fprintf(debugOutput, "%s%d/%d times/cycles",
-			(shown ? ", " : ""), count, cycles);
-		shown = true;
-	}
-	if (shown)
-		fprintf(debugOutput, ":\n");
+		fprintf(debugOutput, "%s:\n", symbol);
 }
 
 /**
@@ -517,7 +502,7 @@ void DebugCpu_Check(void)
 	{
 		Profile_CpuUpdate();
 	}
-	if (LOG_TRACE_LEVEL(TRACE_CPU_DISASM))
+	if (LOG_TRACE_LEVEL((TRACE_CPU_DISASM|TRACE_CPU_SYMBOLS)))
 	{
 		DebugCpu_ShowAddressInfo(M68000_GetPC());
 	}
@@ -548,7 +533,8 @@ void DebugCpu_SetDebugging(void)
 	bCpuProfiling = Profile_CpuStart();
 	nCpuActiveCBs = BreakCond_BreakPointCount(false);
 
-	if (nCpuActiveCBs || nCpuSteps || bCpuProfiling || bHistoryEnabled)
+	if (nCpuActiveCBs || nCpuSteps || bCpuProfiling || bHistoryEnabled
+	    || LOG_TRACE_LEVEL((TRACE_CPU_DISASM|TRACE_CPU_SYMBOLS)))
 		M68000_SetSpecial(SPCFLAG_DEBUGGER);
 	else
 		M68000_UnsetSpecial(SPCFLAG_DEBUGGER);
