@@ -29,7 +29,7 @@ const char Log_fileid[] = "Hatari log.c : " __DATE__ " " __TIME__;
 #include "file.h"
 #include "vdi.h"
 
-
+#if ENABLE_TRACING
 static struct {
 	Uint64 Level;
 	const char *Name;
@@ -101,8 +101,12 @@ TraceOptions[] = {
 	{ TRACE_DSP_STATE	 , "dsp_state" },
 	{ TRACE_DSP_ALL		 , "dsp_all" },
 
+	{ TRACE_DSP_SYMBOLS	 , "dsp_symbols" },
+	{ TRACE_CPU_SYMBOLS	 , "cpu_symbols" },
+
 	{ TRACE_ALL		 , "all" }
 };
+#endif /* ENABLE_TRACING */
 
 
 Uint64	LogTraceFlags = TRACE_NONE;
