@@ -106,6 +106,7 @@ const char DmaSnd_fileid[] = "Hatari dmaSnd.c : " __DATE__ " " __TIME__;
 #include "mfp.h"
 #include "sound.h"
 #include "stMemory.h"
+#include "crossbar.h"
 
 #define TONE_STEPS 13
 
@@ -466,11 +467,11 @@ static inline int DmaSnd_EndOfFrameReached(void)
  * Note: We adjust the volume level of the 8-bit DMA samples to factor
  * 0.75 compared to the PSG sound samples.
  *
- * The following formula: -((256*3/4)/4)/4
+ * The following formula: -((256*3/4)/2)/4
  *
  * Multiply by 256 to convert 8 to 16 bits;
  * DMA sound is 3/4 level of YM sound;
- * Divide by 4 to account for MixBuffer[];
+ * Divide by 2 to account for MixBuffer[];
  * Divide by 4 to account for DmaSnd_LowPassFilter;
  * Multiply DMA sound by -1 because the LMC1992 inverts the signal
  * ( YM sign is +1 :: -1(op-amp) * -1(Lmc1992) ).
@@ -481,7 +482,7 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 {
 	int i;
 	int nBufIdx;
-	Sint8 	MonoByte , LeftByte , RightByte;
+	Sint8 MonoByte , LeftByte , RightByte;
 	unsigned n;
 	Sint64 FreqRatio;
 
@@ -495,22 +496,25 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 
 			switch (microwire.mixing) {
 				case 1:
-					/* YM2149 */
-					MixBuffer[nBufIdx][1]  = MixBuffer[nBufIdx][0] + dma.FrameRight * -((256*3/4)/4)/4;	
-					MixBuffer[nBufIdx][0] += dma.FrameLeft * -((256*3/4)/4)/4;	
+					/* DMA and (YM2149 0 dB) mixing */
+					MixBuffer[nBufIdx][1]  = MixBuffer[nBufIdx][0] + dma.FrameRight * -((256*3/4)/2)/4;	
+					MixBuffer[nBufIdx][0] += dma.FrameLeft * -((256*3/4)/2)/4;	
 					break;
+				case 2:
+					/* DMA only (but DMA is off in that case) */
+					MixBuffer[nBufIdx][0]  = MixBuffer[nBufIdx][1] = 0;
 				default:
-					/* YM2149 - 12dB */
+					/* DMA and (YM2149 -12 dB) mixing */
 					MixBuffer[nBufIdx][0] /= 4;
-					MixBuffer[nBufIdx][1]  = MixBuffer[nBufIdx][0] + dma.FrameRight * -((256*3/4)/4)/4;	
-					MixBuffer[nBufIdx][0] += dma.FrameLeft * -((256*3/4)/4)/4;	
+					MixBuffer[nBufIdx][1]  = MixBuffer[nBufIdx][0] + dma.FrameRight * -((256*3/4)/2)/4;	
+					MixBuffer[nBufIdx][0] += dma.FrameLeft * -((256*3/4)/2)/4;	
 					break;
 			}
 		}
 
 		/* Apply LMC1992 sound modifications (Bass and Treble) */
 		DmaSnd_Apply_LMC ( nMixBufIdx , nSamplesToGenerate );
-		
+
 		return;
 	}
 
@@ -538,17 +542,17 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 
 			switch (microwire.mixing) {
 				case 1:
-					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
+					/* DMA and (YM2149 0 dB) mixing */
+					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/2)/4;
 					break;
 				case 2:
-					/* DMA sound only */
-					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
+					/* DMA only */
+					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/2)/4;
 					break;
 				default:
-					/* DMA and (YM2149 - 12dB) mixing */
-					/* instead of 16462 (-12dB), we approximate by 16384 */
-					MixBuffer[nBufIdx][0] = (dma.FrameLeft * -((256*3/4)/4)/4) +
+					/* DMA and (YM2149 -12 dB) mixing */
+					/* instead of 16462 (-12 dB), we approximate by 16384 */
+					MixBuffer[nBufIdx][0] = (dma.FrameLeft * -((256*3/4)/2)/4) +
 								(((Sint32)MixBuffer[nBufIdx][0] * 16384)/65536);
 					break;
 			}
@@ -586,21 +590,21 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 
 			switch (microwire.mixing) {
 				case 1:
-					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
+					/* DMA and (YM2149 0 dB) mixing */
+					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/2)/4;
+					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/2)/4;
 					break;
 				case 2:
-					/* DMA sound only */
-					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
+					/* DMA only */
+					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/2)/4;
+					MixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/2)/4;
 					break;
 				default:
-					/* DMA and (YM2149 - 12dB) mixing */
-					/* instead of 16462 (-12dB), we approximate by 16384 */
-					MixBuffer[nBufIdx][0] = (dma.FrameLeft * -((256*3/4)/4)/4) +
+					/* DMA and (YM2149 -12 dB) mixing */
+					/* instead of 16462 (-12 dB), we approximate by 16384 */
+					MixBuffer[nBufIdx][0] = (dma.FrameLeft * -((256*3/4)/2)/4) +
 								(((Sint32)MixBuffer[nBufIdx][0] * 16384)/65536);
-					MixBuffer[nBufIdx][1] = (dma.FrameRight * -((256*3/4)/4)/4) +
+					MixBuffer[nBufIdx][1] = (dma.FrameRight * -((256*3/4)/2)/4) +
 								(((Sint32)MixBuffer[nBufIdx][1] * 16384)/65536);
 					break;
 			}
@@ -875,6 +879,12 @@ void DmaSnd_InterruptHandler_Microwire(void)
 {
 	Uint8 i, bit;
 	Uint16 saveData;
+
+	/* If emulated computer is the Falcon, let's the crossbar Microwire code do the job. */
+	if (ConfigureParams.System.nMachineType == MACHINE_FALCON) {
+		Crossbar_InterruptHandler_Microwire();
+		return;
+	}
 	
 	/* How many cycle was this sound interrupt delayed (>= 0) */
 	microwire.pendingCyclesOver += -INT_CONVERT_FROM_INTERNAL ( PendingInterruptCount , INT_CPU_CYCLE );
