@@ -171,9 +171,11 @@ void dsp_core_reset(void)
 	dsp_core.periph[DSP_SPACE_X][DSP_HOST_HSR]=(1<<DSP_HOST_HSR_HTDE);
 
 	/* host port init, cpu side */
-	dsp_core.hostport[CPU_HOST_CVR]=0x12;
-	dsp_core.hostport[CPU_HOST_ISR]=(1<<CPU_HOST_ISR_TRDY)|(1<<CPU_HOST_ISR_TXDE);
-	dsp_core.hostport[CPU_HOST_IVR]=0x0f;
+	dsp_core.hostport[CPU_HOST_ICR] = 0x0;
+	dsp_core.hostport[CPU_HOST_CVR] = 0x12;
+	dsp_core.hostport[CPU_HOST_ISR] = (1<<CPU_HOST_ISR_TRDY)|(1<<CPU_HOST_ISR_TXDE);
+	dsp_core.hostport[CPU_HOST_IVR] = 0x0f;
+	dsp_core.hostport[CPU_HOST_RX0] = 0x0;
 
 	/* SSI registers */
 	dsp_core.periph[DSP_SPACE_X][DSP_SSI_SR]=1<<DSP_SSI_SR_TDE;
@@ -182,7 +184,9 @@ void dsp_core_reset(void)
 	dsp_core.ssi.TX = 0;
 	dsp_core.ssi.RX = 0;
 	dsp_core.ssi.dspPlay_handshakeMode_frame = 0;
-	
+	dsp_core_ssi_configure(DSP_SSI_CRA, 0);
+	dsp_core_ssi_configure(DSP_SSI_CRB, 0);
+
 	/* Other hardware registers */
 	dsp_core.periph[DSP_SPACE_X][DSP_IPR]=0;
 	dsp_core.periph[DSP_SPACE_X][DSP_BCR]=0xffff;
