@@ -19,10 +19,10 @@ extern Uint32 HostScreen_getPitch(void);
 extern Uint32 HostScreen_getWidth(void);
 extern Uint32 HostScreen_getHeight(void);
 extern Uint8 * HostScreen_getVideoramAddress(void);
-extern void HostScreen_setPaletteColor(Uint8 idx, Uint32 red, Uint32 green, Uint32 blue);
+extern SDL_PixelFormat *HostScreen_getFormat(void);
+extern void HostScreen_setPaletteColor(Uint8 idx, Uint8 red, Uint8 green, Uint8 blue);
 extern Uint32 HostScreen_getPaletteColor(Uint8 idx);
-extern void HostScreen_updatePalette(Uint16 colorCount);
-extern Uint32 HostScreen_getColor(Uint32 red, Uint32 green, Uint32 blue);
+extern void HostScreen_updatePalette(int colorCount);
 extern void HostScreen_setWindowSize(int width, int height, int bpp);
 
 #endif
