@@ -27,9 +27,9 @@
 #define NVRAM_TIMEFORMAT 22
 #define NVRAM_DATESEPERATOR 23
 
-#define NVRAM_BOOTDELAY 24
-#define NVRAM_VIDEOMODE 28
-#define NVRAM_MONITOR	29
+#define NVRAM_BOOTDELAY	24
+#define NVRAM_VMODE1	28
+#define NVRAM_VMODE2	29
 
 #define NVRAM_SCSI	30
 
