@@ -1,5 +1,5 @@
 /*
-  Hatari - CreateFloppyController.m
+  Hatari - Shared.h
 
   This file is distributed under the GNU Public License, version 2 or at
   your option any later version. Read the file gpl.txt for details.
@@ -8,6 +8,7 @@
 #import <Cocoa/Cocoa.h>
 
 // Wrapper to run an NSWindow modally
+@protocol NSWindowDelegate;
 @interface ModalWrapper : NSWindowController <NSWindowDelegate>
 {
     IBOutlet NSWindow *modalWindow;
