@@ -24,7 +24,7 @@ const char DlgFloppy_fileid[] = "Hatari dlgFloppy.c : " __DATE__ " " __TIME__;
 #define FLOPPYDLG_IMGDIR      11
 #define FLOPPYDLG_BROWSEIMG   12
 #define FLOPPYDLG_AUTOB       13
-#define FLOPPYDLG_SLOWFLOPPY  14
+#define FLOPPYDLG_FASTFLOPPY  14
 #define FLOPPYDLG_CREATEIMG   15
 #define FLOPPYDLG_PROTOFF     17
 #define FLOPPYDLG_PROTON      18
@@ -49,7 +49,7 @@ static SGOBJ floppydlg[] =
 	{ SGTEXT, 0, 0, 3,10, 58,1, NULL },
 	{ SGBUTTON, 0, 0, 54,9, 8,1, "Browse" },
 	{ SGCHECKBOX, 0, 0, 2,12, 16,1, "Auto insert B" },
-	{ SGCHECKBOX, 0, 0, 2,14, 21,1, "Slow floppy access" },
+	{ SGCHECKBOX, 0, 0, 2,14, 21,1, "Fast floppy access" },
 	{ SGBUTTON, 0, 0, 42,14, 20,1, "Create blank image" },
 	{ SGTEXT, 0, 0, 2,16, 17,1, "Write protection:" },
 	{ SGRADIOBUT, 0, 0, 21,16, 5,1, "Off" },
@@ -212,11 +212,11 @@ void DlgFloppy_Main(void)
 	}
 	floppydlg[FLOPPYDLG_PROTOFF+ConfigureParams.DiskImage.nWriteProtection].state |= SG_SELECTED;
 
-	/* Slow floppy access */
-	if (ConfigureParams.DiskImage.bSlowFloppy)
-		floppydlg[FLOPPYDLG_SLOWFLOPPY].state |= SG_SELECTED;
+	/* Fast floppy access */
+	if (ConfigureParams.DiskImage.FastFloppy)
+		floppydlg[FLOPPYDLG_FASTFLOPPY].state |= SG_SELECTED;
 	else
-		floppydlg[FLOPPYDLG_SLOWFLOPPY].state &= ~SG_SELECTED;
+		floppydlg[FLOPPYDLG_FASTFLOPPY].state &= ~SG_SELECTED;
 
 	/* Draw and process the dialog */
 	do
@@ -270,5 +270,5 @@ void DlgFloppy_Main(void)
 	}
 
 	ConfigureParams.DiskImage.bAutoInsertDiskB = (floppydlg[FLOPPYDLG_AUTOB].state & SG_SELECTED);
-	ConfigureParams.DiskImage.bSlowFloppy = (floppydlg[FLOPPYDLG_SLOWFLOPPY].state & SG_SELECTED);
+	ConfigureParams.DiskImage.FastFloppy = (floppydlg[FLOPPYDLG_FASTFLOPPY].state & SG_SELECTED);
 }
