@@ -14,6 +14,7 @@ const char HDC_fileid[] = "Hatari hdc.c : " __DATE__ " " __TIME__;
 #include "file.h"
 #include "fdc.h"
 #include "hdc.h"
+#include "ioMem.h"
 #include "log.h"
 #include "memorySnapShot.h"
 #include "mfp.h"
@@ -45,9 +46,28 @@ const char HDC_fileid[] = "Hatari hdc.c : " __DATE__ " " __TIME__;
 // #define HDC_VERBOSE           /* display operations */
 // #define HDC_REALLY_VERBOSE    /* display command packets */
 
+#define HDC_ReadInt16(a, i) (((unsigned) a[i] << 8) | a[i + 1])
+#define HDC_ReadInt24(a, i) (((unsigned) a[i] << 16) | ((unsigned) a[i + 1] << 8) | a[i + 2])
+#define HDC_ReadInt32(a, i) (((unsigned) a[i] << 24) | ((unsigned) a[i + 1] << 16) | ((unsigned) a[i + 2] << 8) | a[i + 3])
+
+/**
+ *  Structure representing an ACSI command block.
+ */
+typedef struct {
+  int readCount;    /* count of number of command bytes written */
+  unsigned char target;
+  unsigned char opcode;
+  bool extended;
+
+  int byteCount;             /* count of number of command bytes written */
+  unsigned char command[10];
+  short int returnCode;      /* return code from the HDC operation */
+} HDCOMMAND;
+
 /* HDC globals */
 HDCOMMAND HDCCommand;
 int nPartitions = 0;
+unsigned long hdSize = 0;
 short int HDCSectorCount;
 bool bAcsiEmuOn = false;
 
@@ -59,8 +79,6 @@ static Uint8 nLastError;
 /*
   FDC registers used:
   - FDCSectorCountRegister
-  - DiskControllerStatus_ff8604rd
-  - DMAModeControl_ff8606wr
 */
 
 
@@ -82,22 +100,52 @@ static unsigned char inquiry_bytes[] =
 
 /*---------------------------------------------------------------------*/
 /**
- * Return the file offset of the sector specified in the current
- * ACSI command block.
+ * Return the device specified in the current ACSI command block.
+ */
+static unsigned char HDC_GetDevice(void)
+{
+	return (HDCCommand.command[1] & 0xE0) >> 5;
+}
+
+/**
+ * Return the file offset of the sector specified in the current ACSI command block.
  */
 static unsigned long HDC_GetOffset(void)
 {
-	unsigned long offset;
-
-	/* construct the logical block adress */
-	offset = ((HD_LBA_MSB(HDCCommand) << 16)
-	          |  (HD_LBA_MID(HDCCommand)  << 8)
-	          |  (HD_LBA_LSB(HDCCommand))) ;
-
-	/* return value in bytes */
-	return(offset * 512);
+	/* offset = logical block address * 512 */
+	return HDCCommand.opcode < 0x20?
+		// class 0
+		(HDC_ReadInt24(HDCCommand.command, 1) & 0x1FFFFF) << 9 :
+		// class 1
+		HDC_ReadInt32(HDCCommand.command, 2) << 9;
 }
 
+/**
+ * Return the count specified in the current ACSI command block.
+ */
+static int HDC_GetCount(void)
+{
+	return HDCCommand.opcode < 0x20?
+		// class 0
+		HDCCommand.command[4] :
+		// class1
+		HDC_ReadInt16(HDCCommand.command, 7);
+}
+
+/**
+ * Return the control byte specified in the current ACSI command block.
+ */
+#ifdef HDC_REALLY_VERBOSE
+static unsigned char HDC_GetControl(void)
+{
+	return HDCCommand.opcode < 0x20?
+		// class 0
+		HDCCommand.command[5] :
+		// class1
+		HDCCommand.command[9];
+}
+#endif
+
 
 /*---------------------------------------------------------------------*/
 /**
@@ -134,8 +182,8 @@ static void HDC_Cmd_Inquiry(void)
 	Uint32 nDmaAddr;
 	int count;
 
-	nDmaAddr = FDC_ReadDMAAddress();
-	count = HD_SECTORCOUNT(HDCCommand);
+	nDmaAddr = FDC_GetDMAAddress();
+	count = HDC_GetCount();
 
 #ifdef HDC_VERBOSE
 	fprintf(stderr,"HDC: Inquiry, %i bytes to 0x%x.\n", count, nDmaAddr);
@@ -175,7 +223,7 @@ static void HDC_Cmd_RequestSense(void)
 	fprintf(stderr,"HDC: Request Sense.\n");
 #endif
 
-	nRetLen = HD_SECTORCOUNT(HDCCommand);
+	nRetLen = HDC_GetCount();
 
 	if ((nRetLen < 4 && nRetLen != 0) || nRetLen > 22)
 	{
@@ -192,7 +240,7 @@ static void HDC_Cmd_RequestSense(void)
 		nRetLen = 22;
 	}
 
-	nDmaAddr = FDC_ReadDMAAddress();
+	nDmaAddr = FDC_GetDMAAddress();
 
 	memset(retbuf, 0, nRetLen);
 
@@ -266,14 +314,14 @@ static void HDC_Cmd_ModeSense(void)
 	fprintf(stderr,"HDC: Mode Sense.\n");
 #endif
 
-	nDmaAddr = FDC_ReadDMAAddress();
+	nDmaAddr = FDC_GetDMAAddress();
 
 	if (!STMemory_ValidArea(nDmaAddr, 16))
 	{
 		Log_Printf(LOG_WARN, "HCD mode sense uses invalid RAM range 0x%x+%i\n", nDmaAddr, 16);
 		HDCCommand.returnCode = HD_STATUS_ERROR;
 	}
-	else if (HDCCommand.command[2] == 0 && HD_SECTORCOUNT(HDCCommand) == 0x10)
+	else if (HDCCommand.command[2] == 0 && HDC_GetCount() == 0x10)
 	{
 		size_t blocks;
 		blocks = File_Length(ConfigureParams.HardDisk.szHardDiskImage) / 512;
@@ -338,6 +386,50 @@ static void HDC_Cmd_FormatDrive(void)
 	//FDCSectorCountRegister = 0;
 }
 
+/*---------------------------------------------------------------------*/
+/**
+ * Read capacity of our disk.
+ */
+static void HDC_Cmd_ReadCapacity(void)
+{
+	Uint32 nDmaAddr = FDC_GetDMAAddress();
+
+#ifdef HDC_VERBOSE
+	fprintf(stderr,"Reading 8 bytes capacity data to addr: 0x%x\n", nDmaAddr);
+#endif
+
+	/* seek to the position */
+	if (STMemory_ValidArea(nDmaAddr, 8))
+	{
+		int nSectors = hdSize / 512;
+		STRam[nDmaAddr++] = (nSectors >> 24) & 0xFF;
+		STRam[nDmaAddr++] = (nSectors >> 16) & 0xFF;
+		STRam[nDmaAddr++] = (nSectors >> 8) & 0xFF;
+		STRam[nDmaAddr++] = (nSectors) & 0xFF;
+		STRam[nDmaAddr++] = 0x00;
+		STRam[nDmaAddr++] = 0x00;
+		STRam[nDmaAddr++] = 0x02;
+		STRam[nDmaAddr++] = 0x00;
+
+		/* Update DMA counter */
+		FDC_WriteDMAAddress(nDmaAddr + 8);
+
+		HDCCommand.returnCode = HD_STATUS_OK;
+		nLastError = HD_REQSENS_OK;
+	}
+	else
+	{
+		Log_Printf(LOG_WARN, "HDC capacity read uses invalid RAM range 0x%x+%i\n", nDmaAddr, 8);
+		HDCCommand.returnCode = HD_STATUS_ERROR;
+		nLastError = HD_REQSENS_NOSECTOR;
+	}
+
+	FDC_SetDMAStatus(false);              /* no DMA error */
+	FDC_AcknowledgeInterrupt();
+	bSetLastBlockAddr = false;
+	//FDCSectorCountRegister = 0;
+}
+
 
 /*---------------------------------------------------------------------*/
 /**
@@ -358,20 +450,20 @@ static void HDC_Cmd_WriteSector(void)
 	else
 	{
 		/* write - if allowed */
-		Uint32 nDmaAddr = FDC_ReadDMAAddress();
+		Uint32 nDmaAddr = FDC_GetDMAAddress();
 #ifndef DISALLOW_HDC_WRITE
-		if (STMemory_ValidArea(nDmaAddr, 512*HD_SECTORCOUNT(HDCCommand)))
+		if (STMemory_ValidArea(nDmaAddr, 512*HDC_GetCount()))
 		{
 			n = fwrite(&STRam[nDmaAddr], 512,
-				   HD_SECTORCOUNT(HDCCommand), hd_image_file);
+				   HDC_GetCount(), hd_image_file);
 		}
 		else
 		{
 			Log_Printf(LOG_WARN, "HDC sector write uses invalid RAM range 0x%x+%i\n",
-				   nDmaAddr, 512*HD_SECTORCOUNT(HDCCommand));
+				   nDmaAddr, 512*HDC_GetCount());
 		}
 #endif
-		if (n == HD_SECTORCOUNT(HDCCommand))
+		if (n == HDC_GetCount())
 		{
 			HDCCommand.returnCode = HD_STATUS_OK;
 			nLastError = HD_REQSENS_OK;
@@ -405,7 +497,7 @@ static void HDC_Cmd_ReadSector(void)
 
 #ifdef HDC_VERBOSE
 	fprintf(stderr,"Reading %i sectors from 0x%x to addr: 0x%x\n",
-	        HD_SECTORCOUNT(HDCCommand), nLastBlockAddr, FDC_ReadDMAAddress());
+	        HDC_GetCount(), nLastBlockAddr, FDC_ReadDMAAddress());
 #endif
 
 	/* seek to the position */
@@ -416,19 +508,19 @@ static void HDC_Cmd_ReadSector(void)
 	}
 	else
 	{
-		Uint32 nDmaAddr = FDC_ReadDMAAddress();
-		if (STMemory_ValidArea(nDmaAddr, 512*HD_SECTORCOUNT(HDCCommand)))
+		Uint32 nDmaAddr = FDC_GetDMAAddress();
+		if (STMemory_ValidArea(nDmaAddr, 512*HDC_GetCount()))
 		{
 			n = fread(&STRam[nDmaAddr], 512,
-				   HD_SECTORCOUNT(HDCCommand), hd_image_file);
+				   HDC_GetCount(), hd_image_file);
 		}
 		else
 		{
 			Log_Printf(LOG_WARN, "HDC sector read uses invalid RAM range 0x%x+%i\n",
-				   nDmaAddr, 512*HD_SECTORCOUNT(HDCCommand));
+				   nDmaAddr, 512*HDC_GetCount());
 			n = 0;
 		}
-		if (n == HD_SECTORCOUNT(HDCCommand))
+		if (n == HDC_GetCount())
 		{
 			HDCCommand.returnCode = HD_STATUS_OK;
 			nLastError = HD_REQSENS_OK;
@@ -469,17 +561,24 @@ static void HDC_Cmd_TestUnitReady(void)
 static void HDC_EmulateCommandPacket(void)
 {
 
-	switch(HD_OPCODE(HDCCommand))
+	switch(HDCCommand.opcode)
 	{
 
 	 case HD_TEST_UNIT_RDY:
 		HDC_Cmd_TestUnitReady();
 		break;
 
+	case HD_READ_CAPACITY1:
+		HDC_Cmd_ReadCapacity();
+		break;
+
 	 case HD_READ_SECTOR:
+	 case HD_READ_SECTOR1:
 		HDC_Cmd_ReadSector();
 		break;
+
 	 case HD_WRITE_SECTOR:
+	 case HD_WRITE_SECTOR1:
 		HDC_Cmd_WriteSector();
 		break;
 
@@ -545,39 +644,52 @@ static void HDC_DebugCommandPacket(FILE *hdlogFile)
 	int opcode;
 	static const char *psComNames[] =
 	{
-		"TEST UNIT READY",
-		"REZERO",
-		"???",
-		"REQUEST SENSE",
-		"FORMAT DRIVE",
-		"VERIFY TRACK (?)",
-		"FORMAT TRACK (?)",
-		"REASSIGN BLOCK",
-		"READ SECTOR(S)",
-		"???",
-		"WRITE SECTOR(S)",
-		"SEEK",
-		"???",
-		"CORRECTION",
-		"???",
-		"TRANSLATE",
-		"SET ERROR THRESHOLD",	/* 0x10 */
-		"USAGE COUNTERS",
-		"INQUIRY",
-		"WRITE DATA BUFFER",
-		"READ DATA BUFFER",
-		"MODE SELECT",
-		"???",
-		"???",
-		"EXTENDED READ",
-		"READ TOC",
-		"MODE SENSE",
-		"SHIP",
-		"RECEIVE DIAGNOSTICS",
-		"SEND DIAGNOSTICS"
+		"TEST UNIT READY",         // 0x00
+		"REZERO",                  // 0x01
+		"???",                     // 0x02
+		"REQUEST SENSE",           // 0x03
+		"FORMAT DRIVE",            // 0x04
+		"VERIFY TRACK (?)",        // 0x05
+		"FORMAT TRACK (?)",        // 0x06
+		"REASSIGN BLOCK",          // 0x07
+		"READ SECTOR(S)",          // 0x08
+		"???",                     // 0x09
+		"WRITE SECTOR(S)",         // 0x0A
+		"SEEK",                    // 0x0B
+		"???",                     // 0x0C
+		"CORRECTION",              // 0x0D
+		"???",                     // 0x0E
+		"TRANSLATE",               // 0x0F
+		"SET ERROR THRESHOLD",     // 0x10
+		"USAGE COUNTERS",          // 0x11
+		"INQUIRY",                 // 0x12
+		"WRITE DATA BUFFER",       // 0x13
+		"READ DATA BUFFER",        // 0x14
+		"MODE SELECT",             // 0x15
+		"???",                     // 0x16
+		"???",                     // 0x17
+		"EXTENDED READ",           // 0x18
+		"READ TOC",                // 0x19
+		"MODE SENSE",              // 0x1A
+		"SHIP",                    // 0x1B
+		"RECEIVE DIAGNOSTICS",     // 0x1C
+		"SEND DIAGNOSTICS",        // 0x1D
+		"???",                     // 0x1E
+		"SET TARGET (EXTENDED)",   // 0x1F
+		"???",                     // 0x20
+		"???",                     // 0x21
+		"???",                     // 0x22
+		"???",                     // 0x23
+		"???",                     // 0x24
+		"READ CAPACITY",           // 0x25
+		"???",                     // 0x26
+		"???",                     // 0x27
+		"READ SECTOR(S)",          // 0x28
+		"???",                     // 0x29
+		"WRITE SECTOR(S)",         // 0x2A
 	};
 
-	opcode = HD_OPCODE(HDCCommand);
+	opcode = HDCCommand.opcode;
 
 	fprintf(hdlogFile,"----\n");
 
@@ -590,14 +702,14 @@ static void HDC_DebugCommandPacket(FILE *hdlogFile)
 		fprintf(hdlogFile, "Unknown HDC opcode!! Value = 0x%x\n", opcode);
 	}
 
-	fprintf(hdlogFile, "Target: %i\n", HD_TARGET(HDCCommand));
-	fprintf(hdlogFile, "Device: %i\n", HD_DEVICE(HDCCommand));
+	fprintf(hdlogFile, "Target: %i\n", HDCCommand.target);
+	fprintf(hdlogFile, "Device: %i\n", HDC_GetDevice());
 	fprintf(hdlogFile, "LBA: 0x%lx\n", HDC_GetOffset()/512);
 
-	fprintf(hdlogFile, "Sector count: 0x%x\n", HD_SECTORCOUNT(HDCCommand));
+	fprintf(hdlogFile, "Sector count: 0x%x\n", HDC_GetCount());
 	fprintf(hdlogFile, "HDC sector count: 0x%x\n", HDCSectorCount);
 	//fprintf(hdlogFile, "FDC sector count: 0x%x\n", FDCSectorCountRegister);
-	fprintf(hdlogFile, "Control byte: 0x%x\n", HD_CONTROL(HDCCommand));
+	fprintf(hdlogFile, "Control byte: 0x%x\n", HDC_GetControl());
 }
 #endif
 
@@ -619,9 +731,6 @@ static void HDC_GetInfo(void)
 	long offset;
 	unsigned char hdinfo[HD_PARTITIONTABLE_SIZE];
 	int i;
-#ifdef HDC_VERBOSE
-	unsigned long size;
-#endif
 
 	nPartitions = 0;
 	if (hd_image_file == NULL)
@@ -635,13 +744,10 @@ static void HDC_GetInfo(void)
 		return;
 	}
 
-#ifdef HDC_VERBOSE
-	size = (((unsigned long) hdinfo[0] << 24)
-	        | ((unsigned long) hdinfo[1] << 16)
-	        | ((unsigned long) hdinfo[2] << 8)
-	        | ((unsigned long) hdinfo[3]));
+	hdSize = HDC_ReadInt32(hdinfo, 0);
 
-	fprintf(stderr, "Total disk size %li Mb\n", size>>11);
+#ifdef HDC_VERBOSE
+	fprintf(stderr, "Total disk size %li Mb\n", hdSize>>11);
 	/* flags for each partition entry are zero if they are not valid */
 	fprintf(stderr, "Partition 0 exists?: %s\n", (hdinfo[4] != 0)?"Yes":"No");
 	fprintf(stderr, "Partition 1 exists?: %s\n", (hdinfo[4+12] != 0)?"Yes":"No");
@@ -689,6 +795,7 @@ bool HDC_Init(void)
 	nNumDrives += nPartitions;
 
 	bAcsiEmuOn = true;
+	HDCCommand.readCount = 0;
 	HDCCommand.byteCount = 0;
 
 	printf("Hard drive image %s mounted.\n", filename);
@@ -715,6 +822,36 @@ void HDC_UnInit(void)
 }
 
 
+/*---------------------------------------------------------------------*/
+/**
+ * Reset command status.
+ */
+void HDC_ResetCommandStatus(void)
+{
+	/*HDCCommand.byteCount = 0;*/  /* Not done on real ST? */
+	HDCCommand.returnCode = 0;
+}
+
+
+/**
+ * Get command status.
+ */
+short int HDC_GetCommandStatus(void)
+{
+	return HDCCommand.returnCode;
+}
+
+
+/*---------------------------------------------------------------------*/
+/**
+ * Get sector count.
+ */
+short int HDC_GetSectorCount(void)
+{
+	return HDCSectorCount;
+}
+
+
 /*---------------------------------------------------------------------*/
 /**
  * Process HDC command packets, called when bytes are 
@@ -726,11 +863,24 @@ void HDC_WriteCommandPacket(void)
 	if (!bAcsiEmuOn)
 		return;
 
-	/* command byte sent, store it. */
-	HDCCommand.command[HDCCommand.byteCount] = (DiskControllerWord_ff8604wr&0xFF);
+	/* command byte sent */
+	unsigned char b = IoMem_ReadByte(0xff8605);
+
+	/* Extract target and extended mode early, read acsi opcode */
+	if (HDCCommand.readCount == 0)
+	{
+		HDCCommand.target = ((b & 0xE0) >> 5);
+		HDCCommand.opcode = (b & 0x1F);
+		HDCCommand.extended = (HDCCommand.opcode == 0x1F);
+	}
+	/* In extended mode, the scsi opcode is at position 1 */
+	else if (HDCCommand.extended && HDCCommand.readCount == 1)
+	{
+		HDCCommand.opcode = (b & 0xFF);
+	}
 
 	/* We only support one target with ID 0 */
-	if (HD_TARGET(HDCCommand) != 0)
+	if (HDCCommand.target != 0)
 	{
 		//FDC_SetDMAStatus(true);
 		//FDC_AcknowledgeInterrupt();
@@ -741,17 +891,22 @@ void HDC_WriteCommandPacket(void)
 		return;
 	}
 
-	/* Successfully received one byte, so increase the byte-count */
-	++HDCCommand.byteCount;
+	/* Successfully received one byte, so increase the byte-count, but in extended mode skip the first byte */
+	if (!HDCCommand.extended || HDCCommand.readCount != 0)
+	{
+		HDCCommand.command[HDCCommand.byteCount++] = b;
+	}
+	++HDCCommand.readCount;
 
-	/* have we received a complete 6-byte packet yet? */
-	if (HDCCommand.byteCount >= 6)
+	/* have we received a complete 6-byte class 0 or 10-byte class 1 packet yet? */
+	if ((HDCCommand.opcode < 0x20 && HDCCommand.byteCount >= 6) ||
+		(HDCCommand.opcode < 0x40 && HDCCommand.byteCount >= 10))
 	{
 #ifdef HDC_REALLY_VERBOSE
 		HDC_DebugCommandPacket(stderr);
 #endif
 		/* If it's aimed for our drive, emulate it! */
-		if (HD_DEVICE(HDCCommand) == 0)
+		if (HDC_GetDevice() == 0)
 		{
 			HDC_EmulateCommandPacket();
 		}
@@ -761,6 +916,7 @@ void HDC_WriteCommandPacket(void)
 			HDCCommand.returnCode = HD_STATUS_ERROR;
 		}
 
+		HDCCommand.readCount = 0;
 		HDCCommand.byteCount = 0;
 	}
 	else
