@@ -38,6 +38,15 @@ const char IKBD_fileid[] = "Hatari ikbd.c : " __DATE__ " " __TIME__;
 /* 2011/07/14	[NP]	Don't clear bytes in transit when ACIA_Reset is called ; if a byte is sent to	*/
 /*			the ikbd it should not be cancelled ? FIXME : this would need more tests on a	*/
 /*			real ST (fix Froggies Over The Fence's menu when selecting a demo).		*/
+/* 2011/07/31	[NP]	Don't clear bytes in transit in the ACIA when the IKBD is reset (fix Overdrive	*/
+/*			by Phalanx).									*/
+/* 2011/12/27	[NP]	When sending new bytes while a byte is already in transfer from ACIA to IKBD,	*/
+/*			don't restart the internal TX timer (fix 'Pandemonium Demos' Intro).		*/
+/*			eg : .loop : move.b d0,$fc02.w    btst #1,$fc00.w    beq.s .loop		*/
+/* 2012/01/22	[NP]	Enable both mouse and joystick reporting when commands 0x12 and 0x14 are	*/
+/*			received during the IKBD reset.							*/
+/* 2012/02/26	[NP]	Handle TX interrupt in the ACIA (eg by sending 0xb6 instead of 0x96 after	*/
+/*			resetting the ACIA) (fix the game 'Hades Nebula').				*/
 
 
 #include <time.h>
@@ -85,6 +94,8 @@ static bool bMouseDisabled, bJoystickDisabled;
 static bool bDuringResetCriticalTime, bBothMouseAndJoy;
 static bool bMouseEnabledDuringReset;
 
+static time_t nTimeOffset;			/* Offset between current time and emulated time */
+
 /* ACIA */
 static Uint8 ACIAControlRegister = 0;
 static Uint8 ACIAStatusRegister = ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;  /* Pass when read 0xfffc00 */
@@ -172,6 +183,19 @@ static bool bByteInTransitFromACIA = false;	/* Is a byte being sent from the ACI
     The keyboard ACIA are address 0xfffc000 and 0xfffc02.
     Default parameters are :- 8-bit word, 1 stopbit, no parity, 7812.5 baud; 500KHz/64 (keyboard clock div)
     Default MIDI parameters are are above but :- 31250 baud; 500KHz/16 (MIDI clock div)
+
+
+  Special behaviours during the IKBD reset :
+    If the following commands are received during the reset of the IKBD,
+    the IKBD will go in a special mode and report both mouse and joystick at the same time :
+	0x08 0x14		relative mouse on , joysticks auto
+	0x08 0x0b 0x14		relative mouse on , mouse threshold , joysticks auto (eg Barbarian 1 by Psygnosis)
+	0x12 0x14		disable mouse , joysticks auto (eg Hammerfist)
+	0x12 0x1a		disable mouse , disable joysticks
+
+    In that case mouse and joystick buttons will be reported in a "mouse report" packet
+    and joystick actions (except buttons) will be reported in a "joystick report" packet.
+
 */
 
 /* List of possible keyboard commands, others are seen as NOPs by keyboard processor */
@@ -392,8 +416,12 @@ void IKBD_Reset_ExeMode ( void )
 	IKBD_ExeMode = false;
 
 	Keyboard.BufferHead = Keyboard.BufferTail = 0;	/* flush all queued bytes that would be read in $fffc02 */
+/* [NP] 2011/07/31 FIXME : IKBD reset should not reset acia and clear bytes in transit */
+/* Else, "Overdrive" lock after doing a 68000 "reset" */
+#if 0
 	bByteInTransitToACIA = false;
 	bByteInTransitFromACIA = false;
+#endif
 	// IKBD_AddKeyToKeyboardBuffer(0xF0);		/* Assume OK, return correct code */
 	IKBD_AddKeyToKeyboardBuffer(0xF1);		/* [NP] Dragonnels demo needs this */
 }
@@ -410,6 +438,7 @@ void IKBD_Reset(bool bCold)
 		KeyboardProcessor.bReset = false;
 		if (CycInt_InterruptActive(INTERRUPT_IKBD_RESETTIMER))
 			CycInt_RemovePendingInterrupt(INTERRUPT_IKBD_RESETTIMER);
+		nTimeOffset = 0;
 	}
 
 	KeyboardProcessor.MouseMode = AUTOMODE_MOUSEREL;
@@ -467,6 +496,7 @@ void IKBD_Reset(bool bCold)
 void IKBD_MemorySnapShot_Capture(bool bSave)
 {
 	unsigned int i;
+	time_t nEmuTime;
 
 	/* Save/Restore details */
 	MemorySnapShot_Store(&Keyboard, sizeof(Keyboard));
@@ -502,6 +532,19 @@ void IKBD_MemorySnapShot_Capture(bool bSave)
 		if ( i >= sizeof ( CustomCodeDefinitions ) / sizeof ( CustomCodeDefinitions[0] ) )	/* not found (should not happen) */
 			IKBD_ExeMode = false;			/* turn off exe mode */
 	}
+
+	/* The time offset is special: When 0, the emulated system is running
+	 * with the current time, so we assume that it will also continue with
+	 * real time when the memory snapshot is loaded again. When it is not
+	 * zero, the program (like the game "Zombi") might expect a certain
+	 * system date. In this case we initialize the nTimeOffset during
+	 * loading so that it maches this system date again. */
+	nEmuTime = (nTimeOffset != 0) ? (time(NULL) - nTimeOffset) : 0;
+	MemorySnapShot_Store(&nEmuTime, sizeof(nEmuTime));
+	if (!bSave)
+	{
+		nTimeOffset = (nEmuTime != 0) ? (time(NULL) - nEmuTime) : 0;
+	}
 }
 
 
@@ -860,6 +903,7 @@ static void IKBD_SendCursorMousePacket(void)
 	/* Limit to '10' loops as host mouse cursor might have a VERY poor quality. */
 	/* Eg, a single mouse movement on and ST gives delta's of '1', mostly, */
 	/* but host mouse might go as high as 20+! */
+//fprintf ( stderr , "key %d %d %d %d %d %d\n" , KeyboardProcessor.Mouse.DeltaX , KeyboardProcessor.Mouse.DeltaY, Keyboard.bOldLButtonDown,Keyboard.bLButtonDown, Keyboard.bOldRButtonDown,Keyboard.bRButtonDown );
 	while ( (i<10) && ((KeyboardProcessor.Mouse.DeltaX!=0) || (KeyboardProcessor.Mouse.DeltaY!=0)
 	                   || (!IKBD_ButtonsEqual(Keyboard.bOldLButtonDown,Keyboard.bLButtonDown)) || (!IKBD_ButtonsEqual(Keyboard.bOldRButtonDown,Keyboard.bRButtonDown))) )
 	{
@@ -1028,8 +1072,8 @@ static void IKBD_CheckResetDisableBug(void)
 			KeyboardProcessor.JoystickMode = AUTOMODE_JOYSTICK;
 			bBothMouseAndJoy = true;
 
-			LOG_TRACE(TRACE_IKBD_ALL, "IKBD Mouse+Joystick disabled "
-			          "during RESET. Revert.\n");
+			LOG_TRACE(TRACE_IKBD_ALL, "ikbd cancel commands 0x12 and 0x1a received during reset,"
+				" enabling joystick and mouse reporting at the same time\n" );
 		}
 	}
 }
@@ -1042,6 +1086,9 @@ static void IKBD_CheckResetDisableBug(void)
  */
 void IKBD_InterruptHandler_ResetTimer(void)
 {
+	LOG_TRACE(TRACE_IKBD_CMDS, "ikbd reset timer completed, resuming ikbd processing VBLs=%i framecyc=%i\n",
+	          nVBLs, Cycles_GetCounter(CYCLES_COUNTER_VIDEO));
+
 	/* Remove this interrupt from list and re-order */
 	CycInt_AcknowledgeInterrupt();
 
@@ -1073,7 +1120,7 @@ void IKBD_InterruptHandler_ResetTimer(void)
  */
 static void IKBD_Cmd_Reset(void)
 {
-	LOG_TRACE(TRACE_IKBD_CMDS, "IKBD_Cmd_Reset. VBLs=%i framecyc=%i\n",
+	LOG_TRACE(TRACE_IKBD_CMDS, "IKBD_Cmd_Reset VBLs=%i framecyc=%i\n",
 	          nVBLs, Cycles_GetCounter(CYCLES_COUNTER_VIDEO));
 
 	/* Check for error series of bytes, eg 0x80,0x01 */
@@ -1110,7 +1157,7 @@ static void IKBD_Cmd_Reset(void)
 		bBothMouseAndJoy = false;
 		bMouseEnabledDuringReset = false;
 
-		LOG_TRACE(TRACE_IKBD_ALL, "IKBD reset done.\n");
+		LOG_TRACE(TRACE_IKBD_ALL, "ikbd reset done, starting reset timer\n");
 	}
 	/* else if not 0x80,0x01 just ignore */
 }
@@ -1392,14 +1439,23 @@ static void IKBD_Cmd_ReturnJoystickAuto(void)
 	KeyboardProcessor.JoystickMode = AUTOMODE_JOYSTICK;
 	KeyboardProcessor.MouseMode = AUTOMODE_OFF;
 
-	/* If mouse was also enabled within time of a reset it isn't disabled now!
-	 * (Used by the game Barbarian 1 by Psygnosis for example) */
-	if (bMouseEnabledDuringReset)
+	/* If mouse was also enabled within time of a reset (0x08 command) it isn't disabled now!
+	 * (used by the game Barbarian 1 by Psygnosis for example) */
+	if ( bDuringResetCriticalTime && bMouseEnabledDuringReset )
 	{
 		KeyboardProcessor.MouseMode = AUTOMODE_MOUSEREL;
 		bBothMouseAndJoy = true;
-		LOG_TRACE(TRACE_IKBD_ALL, "IKBD joystick and mouse "
-		          "enabled during RESET. Mouse not disabled!\n");
+		LOG_TRACE(TRACE_IKBD_ALL, "ikbd commands 0x08 and 0x14 received during reset,"
+			" enabling joystick and mouse reporting at the same time\n" );
+	}
+	/* If mouse was disabled during the reset (0x12 command) it is enabled again
+	 * (used by the game Hammerfist for example) */
+	else if ( bDuringResetCriticalTime && bMouseDisabled )
+	{
+		KeyboardProcessor.MouseMode = AUTOMODE_MOUSEREL;
+		bBothMouseAndJoy = true;
+		LOG_TRACE(TRACE_IKBD_ALL, "ikbd commands 0x12 and 0x14 received during reset,"
+			" enabling joystick and mouse reporting at the same time\n" );
 	}
 
 	/* This command resets the internally previously stored joystick states */
@@ -1534,7 +1590,27 @@ static void IKBD_Cmd_DisableJoysticks(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
+/**
+ * Convert value from 2-digit BCD.
+ */
+static Uint16 IKBD_FromBCD(Uint8 Value)
+{
+	return ((Value >> 4) * 10) | (Value & 0x0f);
+}
+
+
+/**
+ * Convert value to 2-digit BCD.
+ * Note: TOS 2.0x requires BCD conversion with overflow, so the decade
+ * is not calculated modulo 10, or it will end up in the year 2039
+ * instead...
+ */
+static Uint8 IKBD_ToBCD(Uint16 Value)
+{
+	return ((Value / 10) << 4) | (Value % 10);
+}
+
+
 /**
  * TIME-OF-DAY CLOCK SET
  *
@@ -1548,24 +1624,25 @@ static void IKBD_Cmd_DisableJoysticks(void)
  */
 static void IKBD_Cmd_SetClock(void)
 {
-	LOG_TRACE(TRACE_IKBD_CMDS, "IKBD_Cmd_SetClock\n");
+	struct tm NewTime;
+
+	LOG_TRACE(TRACE_IKBD_CMDS,
+		  "IKBD_Cmd_SetClock: %02x %02x %02x %02x %02x %02x\n",
+		  Keyboard.InputBuffer[1], Keyboard.InputBuffer[2],
+		  Keyboard.InputBuffer[3], Keyboard.InputBuffer[4],
+		  Keyboard.InputBuffer[5], Keyboard.InputBuffer[6]);
+
+	NewTime.tm_year = IKBD_FromBCD(Keyboard.InputBuffer[1]);
+	NewTime.tm_mon = IKBD_FromBCD(Keyboard.InputBuffer[2]) - 1;
+	NewTime.tm_mday = IKBD_FromBCD(Keyboard.InputBuffer[3]);
+	NewTime.tm_hour = IKBD_FromBCD(Keyboard.InputBuffer[4]);
+	NewTime.tm_min = IKBD_FromBCD(Keyboard.InputBuffer[5]);
+	NewTime.tm_sec = IKBD_FromBCD(Keyboard.InputBuffer[6]);
+
+	nTimeOffset = time(NULL) - mktime(&NewTime);
 }
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * Convert value to 2-digit BCD.
- * Note: TOS 2.0x requires BCD conversion with overflow, so the decade
- * is not calculated modulo 10, or it will end up in the year 2039
- * instead...
- */
-static Uint8 IKBD_ConvertToBCD(Uint16 Value)
-{
-	return ((Value/10)<<4) | (Value%10);
-}
-
-
-/*-----------------------------------------------------------------------*/
 /**
  * INTERROGATE TIME-OF-DAT CLOCK
  *
@@ -1585,18 +1662,18 @@ static void IKBD_Cmd_ReadClock(void)
 	time_t nTimeTicks;
 
 	/* Get system time */
-	nTimeTicks = time(NULL);
+	nTimeTicks = time(NULL) - nTimeOffset;
 	SystemTime = localtime(&nTimeTicks);
 
 	/* Return packet */
 	IKBD_AddKeyToKeyboardBufferWithDelay(0xFC, 32000);
 	/* Return time-of-day clock as yy-mm-dd-hh-mm-ss as BCD */
-	IKBD_AddKeyToKeyboardBuffer(IKBD_ConvertToBCD(SystemTime->tm_year));  /* yy - year (2 least significant digits) */
-	IKBD_AddKeyToKeyboardBuffer(IKBD_ConvertToBCD(SystemTime->tm_mon+1)); /* mm - Month */
-	IKBD_AddKeyToKeyboardBuffer(IKBD_ConvertToBCD(SystemTime->tm_mday));  /* dd - Day */
-	IKBD_AddKeyToKeyboardBuffer(IKBD_ConvertToBCD(SystemTime->tm_hour));  /* hh - Hour */
-	IKBD_AddKeyToKeyboardBuffer(IKBD_ConvertToBCD(SystemTime->tm_min));   /* mm - Minute */
-	IKBD_AddKeyToKeyboardBuffer(IKBD_ConvertToBCD(SystemTime->tm_sec));   /* ss - Second */
+	IKBD_AddKeyToKeyboardBuffer(IKBD_ToBCD(SystemTime->tm_year));  /* yy - year (2 least significant digits) */
+	IKBD_AddKeyToKeyboardBuffer(IKBD_ToBCD(SystemTime->tm_mon+1)); /* mm - Month */
+	IKBD_AddKeyToKeyboardBuffer(IKBD_ToBCD(SystemTime->tm_mday));  /* dd - Day */
+	IKBD_AddKeyToKeyboardBuffer(IKBD_ToBCD(SystemTime->tm_hour));  /* hh - Hour */
+	IKBD_AddKeyToKeyboardBuffer(IKBD_ToBCD(SystemTime->tm_min));   /* mm - Minute */
+	IKBD_AddKeyToKeyboardBuffer(IKBD_ToBCD(SystemTime->tm_sec));   /* ss - Second */
 
 	LOG_TRACE(TRACE_IKBD_CMDS, "IKBD_Cmd_ReadClock\n");
 }
@@ -1952,60 +2029,70 @@ Uint16 IKBD_GetByteFromACIA(void)
 	ACIAStatusRegister &= ~(ACIA_STATUS_REGISTER__RX_BUFFER_FULL | ACIA_STATUS_REGISTER__INTERRUPT_REQUEST | ACIA_STATUS_REGISTER__OVERRUN_ERROR);
 
 	/* GPIP I4 - General Purpose Pin Keyboard/MIDI interrupt */
-	MFP_GPIP |= 0x10;
+	MFP_GPIP |= 0x10;				/* clear IRQ signal */
 	return ACIAByte;  /* Return byte from keyboard */
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * This interrupt handler is used to simulate a correct number of CPU cycles
+ * These interrupt handlers are used to simulate a correct number of CPU cycles
  * when sending a byte from the ACIA to the ikdb or when receiving a byte
  * from the ikbd to the ACIA (because ACIA serial transfers are running
  * at 7812.5 bps, bytes should not be available immediatly as it would break
  * some programs)
- * - Byte received in the ACIA from the keyboard processor : store byte for read
+ * - Byte received in the ACIA from the keyboard processor (RX) : store byte for read
  *   from $fffc02 and schedule MFP interrupt to be triggered just a few cycles after.
- * - Byte sent from the ACIA to the keyboard processor : pass the byte to
+ * - Byte sent from the ACIA to the keyboard processor (TX) : pass the byte to
  *   IKBD_SendByteToKeyboardProcessor and set TX_BUFFER_EMPTY in status register.
  */
-void IKBD_InterruptHandler_ACIA(void)
+void IKBD_InterruptHandler_ACIA_RX(void)
 {
 	/* Remove this interrupt from list and re-order */
 	CycInt_AcknowledgeInterrupt();
 
-	if ( bByteInTransitToACIA )
+	/* Copy keyboard byte, ready for read from $fffc02 */
+	ACIAByte = Keyboard.Buffer[Keyboard.BufferHead++];
+	Keyboard.BufferHead &= KEYBOARD_BUFFER_MASK;
+
+	/* Did we get an over-run? Ie byte has arrived from keyboard processor BEFORE CPU has read previous one from ACIA */
+	if (ACIAStatusRegister&ACIA_STATUS_REGISTER__RX_BUFFER_FULL)
+		ACIAStatusRegister |= ACIA_STATUS_REGISTER__OVERRUN_ERROR;  /* Set over-run */
+//fprintf ( stderr , "int acia %x %x\n" , ACIAByte, ACIAStatusRegister );
+
+	/* ACIA buffer is now full */
+	ACIAStatusRegister |= ACIA_STATUS_REGISTER__RX_BUFFER_FULL;
+	/* Signal interrupt pending */
+	ACIAStatusRegister |= ACIA_STATUS_REGISTER__INTERRUPT_REQUEST;
+
+	/* GPIP I4 - General Purpose Pin Keyboard/MIDI interrupt */
+	/* NOTE: GPIP will remain low(0) until keyboard data is read from $fffc02. */
+	MFP_GPIP &= ~0x10;				/* set IRQ signal */
+
+	/* There seems to be a small gap on a real ST between the point in time
+	* the ACIA_STATUS_REGISTER__RX_BUFFER_FULL bit is set and the MFP
+	* interrupt is triggered - for example the "V8 music system" demo
+	* depends on this behaviour. To emulate this, we simply start another
+	* Int which triggers the MFP interrupt later: */
+	CycInt_AddRelativeInterrupt(18, INT_CPU_CYCLE, INTERRUPT_IKBD_MFP);
+}
+
+
+void IKBD_InterruptHandler_ACIA_TX(void)
+{
+	/* Remove this interrupt from list and re-order */
+	CycInt_AcknowledgeInterrupt();
+
+	IKBD_SendByteToKeyboardProcessor(ACIATxDataRegister);  		/* Pass the byte to the keyboard processor */
+	ACIAStatusRegister |= ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;	/* TX buffer is now empty */
+	bByteInTransitFromACIA = false;					/* ready to send another byte */
+
+	/* If TX interrupt is enabled do an IRQ now */
+	if ( ( ACIAControlRegister & 0x60 ) == 0x20 )			/* CR6+CR5 = 01 -> transmit interrupt disabled */
 	{
-		/* Copy keyboard byte, ready for read from $fffc02 */
-		ACIAByte = Keyboard.Buffer[Keyboard.BufferHead++];
-		Keyboard.BufferHead &= KEYBOARD_BUFFER_MASK;
-
-		/* Did we get an over-run? Ie byte has arrived from keyboard processor BEFORE CPU has read previous one from ACIA */
-		if (ACIAStatusRegister&ACIA_STATUS_REGISTER__RX_BUFFER_FULL)
-			ACIAStatusRegister |= ACIA_STATUS_REGISTER__OVERRUN_ERROR;  /* Set over-run */
-
-		/* ACIA buffer is now full */
-		ACIAStatusRegister |= ACIA_STATUS_REGISTER__RX_BUFFER_FULL;
-		/* Signal interrupt pending */
-		ACIAStatusRegister |= ACIA_STATUS_REGISTER__INTERRUPT_REQUEST;
-
-		/* GPIP I4 - General Purpose Pin Keyboard/MIDI interrupt */
-		/* NOTE: GPIP will remain low(0) until keyboard data is read from $fffc02. */
-		MFP_GPIP &= ~0x10;
-
-		/* There seems to be a small gap on a real ST between the point in time
-		* the ACIA_STATUS_REGISTER__RX_BUFFER_FULL bit is set and the MFP
-		* interrupt is triggered - for example the "V8 music system" demo
-		* depends on this behaviour. To emulate this, we simply start another
-		* Int which triggers the MFP interrupt later: */
-		CycInt_AddRelativeInterrupt(18, INT_CPU_CYCLE, INTERRUPT_IKBD_MFP);
-	}
-
-	if ( bByteInTransitFromACIA )
-	{
-		IKBD_SendByteToKeyboardProcessor(ACIATxDataRegister);  		/* Pass the byte to the keyboard processor */
-		ACIAStatusRegister |= ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;	/* TX buffer is now empty */
-		bByteInTransitFromACIA = false;					/* ready to send another byte */
+		/* NOTE: GPIP will remain low(0) until byte is written to $fffc02. */
+		MFP_GPIP &= ~0x10;					/* set IRQ signal */
+		MFP_InputOnChannel(MFP_ACIA_BIT, MFP_IERB, &MFP_IPRB);
 	}
 }
 
@@ -2015,6 +2102,7 @@ void IKBD_InterruptHandler_ACIA(void)
  */
 void IKBD_InterruptHandler_MFP(void)
 {
+//fprintf ( stderr , "int mfp %x %x\n" , ACIAByte, ACIAStatusRegister );
 	/* Remove this interrupt from list and re-order */
 	CycInt_AcknowledgeInterrupt();
 
@@ -2043,7 +2131,7 @@ static void IKBD_SendByteToACIA(int nAciaCycles)
 	if (!bByteInTransitToACIA)
 	{
 		/* Send byte to ACIA */
-		CycInt_AddRelativeInterrupt(nAciaCycles, INT_CPU_CYCLE, INTERRUPT_IKBD_ACIA);
+		CycInt_AddRelativeInterrupt(nAciaCycles, INT_CPU_CYCLE, INTERRUPT_IKBD_ACIA_RX);
 		/* Set flag so only transmit one byte at a time */
 		bByteInTransitToACIA = true;
 	}
@@ -2178,22 +2266,32 @@ void IKBD_KeyboardData_ReadByte(void)
  */
 void IKBD_KeyboardControl_WriteByte(void)
 {
+	int FrameCycles, HblCounterVideo, LineCycles;
+
 	/* ACIA registers need wait states - but the value seems to vary in certain cases */
 	M68000_WaitState(8);
 
-	if (LOG_TRACE_LEVEL(TRACE_IKBD_ACIA))
-	{
-		int FrameCycles, HblCounterVideo, LineCycles;
-		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
-		LOG_TRACE_PRINT("ikbd write fffc00 ctrl=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+	Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+	LOG_TRACE(TRACE_IKBD_ACIA, "ikbd write fffc00 ctrl=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 				IoMem[0xfffc00], FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
-	}
 
 	/* [NP] We only handle reset of the ACIA */
 	if ( ( IoMem[0xfffc00] & 0x03 ) == 0x03 )
 		ACIA_Reset();
 
-	/* Nothing... */
+	/* If TX interrupt is enabled (CR6+CR5 go from 00 to 01) and TX buffer is empty, do an IRQ now */
+	if ( ( ( ACIAControlRegister & 0x60 ) == 0x00 )			/* CR6+CR5 = 00 -> transmit interrupt disabled */
+	  && ( ( IoMem[0xfffc00] & 0x60 ) == 0x20 )			/* CR6+CR5 = 01 -> transmit interrupt enabled */
+	  && ( ACIAStatusRegister & ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY ) )
+	{
+		LOG_TRACE(TRACE_IKBD_ACIA, "ikbd write fffc00 ctrl=0x%x enable ACIA TX IRQ video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+				IoMem[0xfffc00], FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+		/* NOTE: GPIP will remain low(0) until byte is written to $fffc02. */
+		MFP_GPIP &= ~0x10;					/* set IRQ signal */
+		MFP_InputOnChannel(MFP_ACIA_BIT, MFP_IERB, &MFP_IPRB);
+	}
+
+	ACIAControlRegister = IoMem[0xfffc00];
 }
 
 /*-----------------------------------------------------------------------*/
@@ -2202,23 +2300,42 @@ void IKBD_KeyboardControl_WriteByte(void)
  */
 void IKBD_KeyboardData_WriteByte(void)
 {
+	int FrameCycles, HblCounterVideo, LineCycles;
+
 	/* ACIA registers need wait states - but the value seems to vary in certain cases */
 	M68000_WaitState(8);
 
-	if (LOG_TRACE_LEVEL(TRACE_IKBD_ACIA))
-	{
-		int FrameCycles, HblCounterVideo, LineCycles;
-		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
-		LOG_TRACE_PRINT("ikbd write fffc02 data=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+	Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+	LOG_TRACE(TRACE_IKBD_ACIA, "ikbd write fffc02 data=0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 				IoMem[0xfffc02], FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+
+	if ( bDuringResetCriticalTime )				/* warn when some byte are sent to the IKBD during its reset */
+	{
+		LOG_TRACE(TRACE_IKBD_ACIA, "ikbd write fffc02 data=0x%x during reset might be ignored video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+				IoMem[0xfffc02],  FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+
 	}
 
 
-	if ( bByteInTransitFromACIA == false )			/* we're not already transfering a byte to the ikbd */
+	if ( bByteInTransitFromACIA == true )			/* we're already transfering a byte to the ikbd */
 	{
-//		IKBD_SendByteToKeyboardProcessor(IoMem[0xfffc02]);  /* Pass our byte to the keyboard processor */
-		ACIATxDataRegister = IoMem[0xfffc02];		/* store the byte that we want to send to the ikbd */
+		LOG_TRACE(TRACE_IKBD_ACIA, "ikbd write fffc02 data=0x%x cancels command 0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+				IoMem[0xfffc02], ACIATxDataRegister, FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
 
+	}
+
+	ACIATxDataRegister = IoMem[0xfffc02];			/* store the byte that we want to send to the ikbd */
+
+	/* A write in TDR clears the TX IRQ */
+	if ( ( ACIAControlRegister & 0x60 ) == 0x20 )		/* CR6+CR5 = 01 -> transmit interrupt enabled */
+		MFP_GPIP |= 0x10;				/* clear IRQ signal */
+
+
+	/* [NP] FIXME 2011/12/27 : when writing constantly in $fffc02, we should replace ACIATxDataRegister */
+	/* but we should not restart INTERRUPT_IKBD_ACIA_TX from the start each time, nor reset TX_BUFFER bit */
+	/* ('Pandemonium Demos' by Chaos). This should be measured on a real ST. */
+	if ( bByteInTransitFromACIA == false )
+	{
 		/* Delay the processing of the byte in IKBD_InterruptHandler_ACIA */
 		/* The delay doesn't seem to be constant, so we add a small random number of max 40 cycles */
 		/* (else some programs get stuck in an endless loop ('Pandemonium Demos' by Chaos) */
@@ -2227,18 +2344,18 @@ void IKBD_KeyboardData_WriteByte(void)
 		// a bug where addr $6122/$6124 are overwritten by the stack, preventing the desktop
 		// to be restored at the correct resolution !
 		// For now, use a delay of 1000 cycles ; need to do complete measures on a real ST for this
-		//CycInt_AddRelativeInterrupt(ACIA_CYCLES+rand()%40, INT_CPU_CYCLE, INTERRUPT_IKBD_ACIA);
+		//CycInt_AddRelativeInterrupt(ACIA_CYCLES+rand()%40, INT_CPU_CYCLE, INTERRUPT_IKBD_ACIA_TX);
 // delay cycles <=1560 ok, > 1570 bad for tos 1.02/1.04
-		CycInt_AddRelativeInterrupt(1000+rand()%40, INT_CPU_CYCLE, INTERRUPT_IKBD_ACIA);
+		CycInt_AddRelativeInterrupt(1000+rand()%40, INT_CPU_CYCLE, INTERRUPT_IKBD_ACIA_TX);
 
 		/* Some games like USS John Young / FOF54 actually check whether the
 		* transmit-buffer-empty bit is really cleared after writing a data
 		* byte to the IKBD, so we have to temporarily clear this bit, too,
 		* although the byte is send immediately to our virtual IKBD. */
 		ACIAStatusRegister &= ~ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY;		/* TX buffer is full */
-
-		bByteInTransitFromACIA = true;
 	}
+
+	bByteInTransitFromACIA = true;
 }
 
 
