@@ -42,6 +42,7 @@ typedef struct
 {
   bool bEnableMicrophone;
   bool bEnableSound;
+  bool bEnableSoundSync;
   int nPlaybackFreq;
   int SdlAudioBufferSize;
   char szYMCaptureFileName[FILENAME_MAX];
@@ -228,7 +229,6 @@ typedef struct
 typedef struct
 {
   bool bEnablePrinting;
-  bool bPrintToFile;
   char szPrintToFileName[FILENAME_MAX];
 } CNF_PRINTER;
 
@@ -262,10 +262,10 @@ typedef enum
 #if ENABLE_WINUAE_CPU
 typedef enum
 {
-  FPU_NONE,
-  FPU_68881,
-  FPU_68882,
-  FPU_CPU
+  FPU_NONE = 0,
+  FPU_68881 = 68881,
+  FPU_68882 = 68882,
+  FPU_CPU = 68040
 } FPUTYPE;
 #endif
 
