@@ -24,7 +24,7 @@ typedef enum
   INTERRUPT_IKBD_ACIA_RX,
   INTERRUPT_IKBD_MFP,
   INTERRUPT_IKBD_AUTOSEND,
-  INTERRUPT_DMASOUND_MICROWIRE,
+  INTERRUPT_DMASOUND_MICROWIRE, /* Used for both STE and Falcon Microwire emulation */
   INTERRUPT_CROSSBAR_25MHZ,
   INTERRUPT_CROSSBAR_32MHZ,
   INTERRUPT_FDC,
