@@ -75,7 +75,7 @@ typedef struct {
 /* 0xfffc00 (read status from ACIA) */
 #define ACIA_STATUS_REGISTER__RX_BUFFER_FULL  0x01
 #define ACIA_STATUS_REGISTER__TX_BUFFER_EMPTY  0x02
-#define ACIA_STATUS_REGISTER__OVERRUN_ERROR    0x40
+#define ACIA_STATUS_REGISTER__OVERRUN_ERROR    0x20
 #define ACIA_STATUS_REGISTER__INTERRUPT_REQUEST  0x80
 
 extern KEYBOARD_PROCESSOR KeyboardProcessor;
@@ -86,7 +86,8 @@ extern void IKBD_Reset_ExeMode ( void );
 extern void IKBD_Reset(bool bCold);
 extern void IKBD_MemorySnapShot_Capture(bool bSave);
 extern void IKBD_InterruptHandler_ResetTimer(void);
-extern void IKBD_InterruptHandler_ACIA(void);
+extern void IKBD_InterruptHandler_ACIA_RX(void);
+extern void IKBD_InterruptHandler_ACIA_TX(void);
 extern void IKBD_InterruptHandler_MFP(void);
 extern void IKBD_InterruptHandler_AutoSend(void);
 
