@@ -13,7 +13,7 @@ extern bool bBiosIntercept;
 extern bool AviRecordOnStartup;
 
 extern Uint32 Opt_GetNoParachuteFlag(void);
-extern bool Opt_ParseParameters(int argc, const char *argv[]);
+extern bool Opt_ParseParameters(int argc, const char * const argv[]);
 extern char *Opt_MatchOption(const char *text, int state);
 
 #endif /* HATARI_OPTIONS_H */
