@@ -61,8 +61,8 @@ typedef struct
   char *txt;            /* Text string */
 }  SGOBJ;
 
-int sdlgui_fontwidth;	/* Width of the actual font */
-int sdlgui_fontheight;	/* Height of the actual font */
+extern int sdlgui_fontwidth;	/* Width of the actual font */
+extern int sdlgui_fontheight;	/* Height of the actual font */
 
 extern int SDLGui_Init(void);
 extern int SDLGui_UnInit(void);
