@@ -43,6 +43,7 @@ typedef		yms16			ymsample;	/* Output samples are mono 16bits signed PCM */
 
 #define YM_LINEAR_MIXING		1		/* Use ymout1c5bit[] to build ymout5[] */
 #define YM_TABLE_MIXING			2		/* Use volumetable_original to build ymout5[] */
+#define YM_MODEL_MIXING			3		/* Use circuit analysis model to build ymout5[] */
 
 extern int	YmVolumeMixing;
 extern bool	UseLowPassFilter;
@@ -58,6 +59,8 @@ extern bool Sound_BeginRecording(char *pszCaptureFileName);
 extern void Sound_EndRecording(void);
 extern bool Sound_AreWeRecording(void);
 extern void Sound_SetYmVolumeMixing(void);
+extern ymsample Subsonic_IIR_HPF_Left(ymsample x0);
+extern ymsample Subsonic_IIR_HPF_Right(ymsample x0);
 
 
 #endif  /* HATARI_SOUND_H */
