@@ -210,17 +210,24 @@ void M68000_Init(void)
  */
 void M68000_Reset(bool bCold)
 {
-	/* Clear registers */
+#if ENABLE_WINUAE_CPU
 	if (bCold)
 	{
+		/* Clear registers, but we need to keep SPCFLAG_MODE_CHANGE and SPCFLAG_BRK unchanged */
+		int spcFlags = regs.spcflags & (SPCFLAG_MODE_CHANGE | SPCFLAG_BRK);
+		memset(&regs, 0, sizeof(regs));
+		regs.spcflags = spcFlags;
+	}
+	/* Now reset the WINUAE CPU core */
+	/* Laurent : for now, using parameter 0 (Hard reset), but some other parameters can be used here (see newcpu.c) */
+	m68k_reset(0);
+#else /* UAE CPU core */
+	if (bCold)
+	{
+		/* Clear registers */
 		memset(&regs, 0, sizeof(regs));
 	}
-
-	/* Now directly reset the UAE CPU core: */
-	/* Laurent : for now, using parameter 0, but some other parameters can be used here (see newcpu.c) */
-#if ENABLE_WINUAE_CPU
-	m68k_reset(0);
-#else
+	/* Now reset the UAE CPU core */
 	m68k_reset();
 #endif
 	BusMode = BUS_MODE_CPU;
@@ -272,6 +279,8 @@ void M68000_CheckCpuSettings(void)
 	changed_prefs.cpu_compatible = ConfigureParams.System.bCompatibleCpu;
 
 #if ENABLE_WINUAE_CPU
+	/* WinUAE core uses cpu_model instead of cpu_level, so we've got to
+	 * convert these values here: */
 	switch (changed_prefs.cpu_level) {
 		case 0 : changed_prefs.cpu_model = 68000; break;
 		case 1 : changed_prefs.cpu_model = 68010; break;
@@ -281,6 +290,7 @@ void M68000_CheckCpuSettings(void)
 		case 5 : changed_prefs.cpu_model = 68060; break;
 		default: fprintf (stderr, "Init680x0() : Error, cpu_level unknown\n");
 	}
+	currprefs.cpu_level = changed_prefs.cpu_level;
 
 	changed_prefs.address_space_24 = ConfigureParams.System.bAddressSpace24;
 	changed_prefs.cpu_cycle_exact = ConfigureParams.System.bCycleExactCpu;
