@@ -292,6 +292,8 @@ void Main_WaitOnVbl(void)
 	if ( DestTicks == 0 )					/* first call, init DestTicks */
 		DestTicks = CurrentTicks + FrameDuration_micro;
 
+	DestTicks += pulse_swallowing_count; /* audio.c - Audio_CallBack() */
+
 	nDelay = DestTicks - CurrentTicks;
 
 	/* Do not wait if we are in fast forward mode or if we are totally out of sync */
@@ -585,6 +587,7 @@ static void Main_Init(void)
 	Printer_Init();
 	RS232_Init();
 	Midi_Init();
+	Control_CheckUpdates();       /* enable window embedding? */
 	Screen_Init();
 	Main_SetTitle(NULL);
 	HostScreen_Init();
@@ -706,7 +709,7 @@ static void Main_StatusbarSetup(void)
 	{
 		char message[24], *keyname;
 #ifdef _MUDFLAP
-		__mf_register(name, 32, __MF_TYPE_GUESS, "SDL keyname");
+		__mf_register((void*)name, 32, __MF_TYPE_GUESS, "SDL keyname");
 #endif
 		keyname = Str_ToUpper(strdup(name));
 		snprintf(message, sizeof(message), "Press %s for Options", keyname);
@@ -750,11 +753,13 @@ int main(int argc, char *argv[])
 	Win_OpenCon();
 #endif
 
-	/* Needed on maemo but useful also with normal X11 window managers
-	 * for window grouping when you have multiple Hatari SDL windows open
-	 */
 #if HAVE_SETENV
+	/* Needed on maemo but useful also with normal X11 window managers for
+	 * window grouping when you have multiple Hatari SDL windows open */
 	setenv("SDL_VIDEO_X11_WMCLASS", "hatari", 1);
+
+	/* Needed for proper behavior of Caps Lock on some systems */
+	setenv("SDL_DISABLE_LOCK_KEYS", "1", 1);
 #endif
 
 	/* Init emulator system */
