@@ -129,6 +129,7 @@ enum {
 	OPT_MICROPHONE,		/* sound options */
 	OPT_SOUND,
 	OPT_SOUNDBUFFERSIZE,
+	OPT_SOUNDSYNC,
 	OPT_YM_MIXING,
 	OPT_DEBUG,		/* debug options */
 	OPT_BIOSINTERCEPT,
@@ -345,6 +346,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "Sound frequency (x=off/6000-50066, off=fastest)" },
 	{ OPT_SOUNDBUFFERSIZE,   NULL, "--sound-buffer-size",
 	  "<x>", "Sound buffer size in ms (x=0/10-100, 0=default)" },
+	{ OPT_SOUNDSYNC,   NULL, "--sound-sync",
+	  "<bool>", "Sound synchronized emulation (on|off, off=default)" },
 	{ OPT_YM_MIXING,   NULL, "--ym-mixing",
 	  "<x>", "YM sound mixing method (x=linear/table/model)" },
 
@@ -1432,7 +1435,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			else if (strcasecmp(argv[i], "falcon") == 0)
 			{
 #if ENABLE_DSP_EMU
-		ConfigureParams.System.nDSPType = DSP_TYPE_EMU;
+				ConfigureParams.System.nDSPType = DSP_TYPE_EMU;
 #endif
 				ConfigureParams.System.nMachineType = MACHINE_FALCON;
 				ConfigureParams.System.nCpuLevel = 3;
@@ -1442,6 +1445,17 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			{
 				return Opt_ShowError(OPT_MACHINE, argv[i], "Unknown machine type");
 			}
+#if ENABLE_WINUAE_CPU
+			ConfigureParams.System.bMMU = false;	/* does this affect also other than 040 CPUs? */
+			ConfigureParams.System.bAddressSpace24 = true;
+			if (strcasecmp(argv[i], "tt") == 0)
+			{
+				ConfigureParams.System.bCompatibleFPU = true;
+				ConfigureParams.System.n_FPUType = FPU_68882;
+			} else {
+				ConfigureParams.System.n_FPUType = FPU_NONE;	/* TODO: or leave it as-is? */
+			}
+#endif
 			bLoadAutoSave = false;
 			break;
 
@@ -1539,6 +1553,10 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				}
 			ConfigureParams.Sound.SdlAudioBufferSize = temp;
 			break;
+
+		case OPT_SOUNDSYNC:
+			ok = Opt_Bool(argv[++i], OPT_SOUNDSYNC, &ConfigureParams.Sound.bEnableSoundSync);
+			break;
 			
 		case OPT_MICROPHONE:
 			ok = Opt_Bool(argv[++i], OPT_MICROPHONE, &ConfigureParams.Sound.bEnableMicrophone);
