@@ -57,6 +57,11 @@
 /*			always return 0xff (tested on STF).				*/
 /*			When PSGRegisterSelect > 15, reads to $ff8800 also return 0xff.	*/
 /* 2009/01/24	[NP]	Remove redundant test, as movep implies SIZE_BYTE access.	*/
+/* 2011/10/30	[NP]	There's a special case when reading a register from $ff8800 :	*/
+/*			if the register number was not changed since the last write,	*/
+/*			then we must return the value that was written to $ff8802	*/
+/*			without masking the unused bit (fix the game Murders In Venice,	*/
+/*			which expects to read $10 from reg 3).				*/
 
 
 /* Emulating wait states when accessing $ff8800/01/02/03 with different 'move' variants	*/
@@ -124,8 +129,9 @@ const char PSG_fileid[] = "Hatari psg.c : " __DATE__ " " __TIME__;
 #include "mfp.h"
 
 
-Uint8 PSGRegisterSelect;        /* 0xff8800 (read/write) */
-Uint8 PSGRegisters[16];         /* Register in PSG, see PSG_REG_xxxx */
+Uint8 PSGRegisterSelect;        /* Write to 0xff8800 sets the register number used in read/write accesses */
+Uint8 PSGRegisterReadData;	/* Value returned when reading from 0xff8800 */
+Uint8 PSGRegisters[MAX_PSG_REGISTERS]; /* Registers in PSG, see PSG_REG_xxxx */
 
 static unsigned int LastStrobe=0; /* Falling edge of Strobe used for printer */
 
@@ -136,8 +142,25 @@ static unsigned int LastStrobe=0; /* Falling edge of Strobe used for printer */
  */
 void PSG_Reset(void)
 {
+        int     i;
+
+	if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
+	{
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		LOG_TRACE_PRINT("ym reset video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
+		                FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
+	}
+
 	PSGRegisterSelect = 0;
+	PSGRegisterReadData = 0;
 	memset(PSGRegisters, 0, sizeof(PSGRegisters));
+	PSGRegisters[PSG_REG_IO_PORTA] = 0xff;			/* no drive selected + side 0 after a reset */
+
+	/* Update sound's emulation registers */
+        for ( i=0 ; i < NUM_PSG_SOUND_REGISTERS; i++ )
+		Sound_WriteReg ( i , 0 );
+
 	LastStrobe=0;
 }
 
@@ -150,6 +173,7 @@ void PSG_MemorySnapShot_Capture(bool bSave)
 {
 	/* Save/Restore details */
 	MemorySnapShot_Store(&PSGRegisterSelect, sizeof(PSGRegisterSelect));
+	MemorySnapShot_Store(&PSGRegisterReadData, sizeof(PSGRegisterReadData));
 	MemorySnapShot_Store(PSGRegisters, sizeof(PSGRegisters));
 	MemorySnapShot_Store(&LastStrobe, sizeof(LastStrobe));
 }
@@ -168,6 +192,11 @@ void PSG_Set_SelectRegister(Uint8 val)
 	/* read/write to ff8802 when PSGRegisterSelect >= 16 */
 	PSGRegisterSelect = val;
 
+	/* When address register is changed, a read from $ff8800 should */
+	/* return the masked value of the register. We set the value here */
+	/* to be returned in case PSG_Get_DataRegister is called */
+	PSGRegisterReadData = PSGRegisters[PSGRegisterSelect];
+
 	if (LOG_TRACE_LEVEL(TRACE_PSG_WRITE))
 	{
 		int FrameCycles, HblCounterVideo, LineCycles;
@@ -186,37 +215,37 @@ void PSG_Set_SelectRegister(Uint8 val)
 Uint8 PSG_Get_DataRegister(void)
 {
 	/* Is a valid PSG register currently selected ? */
-	if ( PSGRegisterSelect >= 16 )
+	if ( PSGRegisterSelect >= MAX_PSG_REGISTERS )
 		return 0xff;				/* not valid, return 0xff */
 
-	if (PSGRegisterSelect == 14)
+	if (PSGRegisterSelect == PSG_REG_IO_PORTA)
 	{
 		/* Second parallel port joystick uses centronics strobe bit as fire button: */
 		if (ConfigureParams.Joysticks.Joy[JOYID_PARPORT2].nJoystickMode != JOYSTICK_DISABLED)
 		{
 			if (Joy_GetStickData(JOYID_PARPORT2) & 0x80)
-				PSGRegisters[14] &= ~32;
+				PSGRegisters[PSG_REG_IO_PORTA] &= ~32;
 			else
-				PSGRegisters[14] |= 32;
+				PSGRegisters[PSG_REG_IO_PORTA] |= 32;
 		}
 	}
-	else if (PSGRegisterSelect == 15)
+	else if (PSGRegisterSelect == PSG_REG_IO_PORTB)
 	{
 		/* PSG register 15 is parallel port data register - used by parallel port joysticks: */
 		if (ConfigureParams.Joysticks.Joy[JOYID_PARPORT1].nJoystickMode != JOYSTICK_DISABLED)
 		{
-			PSGRegisters[15] &= 0x0f;
-			PSGRegisters[15] |= ~Joy_GetStickData(JOYID_PARPORT1) << 4;
+			PSGRegisters[PSG_REG_IO_PORTB] &= 0x0f;
+			PSGRegisters[PSG_REG_IO_PORTB] |= ~Joy_GetStickData(JOYID_PARPORT1) << 4;
 		}
 		if (ConfigureParams.Joysticks.Joy[JOYID_PARPORT2].nJoystickMode != JOYSTICK_DISABLED)
 		{
-			PSGRegisters[15] &= 0xf0;
-			PSGRegisters[15] |= ~Joy_GetStickData(JOYID_PARPORT2) & 0x0f;
+			PSGRegisters[PSG_REG_IO_PORTB] &= 0xf0;
+			PSGRegisters[PSG_REG_IO_PORTB] |= ~Joy_GetStickData(JOYID_PARPORT2) & 0x0f;
 		}
 	}
 
 	/* Read data last selected by register */
-	return PSGRegisters[PSGRegisterSelect];
+	return PSGRegisterReadData;
 }
 
 
@@ -236,12 +265,16 @@ void PSG_Set_DataRegister(Uint8 val)
 	}
 
 	/* Is a valid PSG register currently selected ? */
-	if ( PSGRegisterSelect >= 16 )
+	if ( PSGRegisterSelect >= MAX_PSG_REGISTERS )
 		return;					/* not valid, ignore write and do nothing */
 
 	/* Create samples up until this point with current values */
 	Sound_Update(false);
 
+	/* When a read is made from $ff8800 without changing PSGRegisterSelect, we should return */
+	/* the non masked value. */
+	PSGRegisterReadData = val;			/* store non masked value for PSG_Get_DataRegister */
+
 	/* Copy value to PSGRegisters[] */
 	PSGRegisters[PSGRegisterSelect] = val;
 
@@ -255,7 +288,6 @@ void PSG_Set_DataRegister(Uint8 val)
 	  PSGRegisters[PSGRegisterSelect] &= 0x1f;	/* only keep bits 0 - 4 */
 
 
-
 	if ( PSGRegisterSelect < NUM_PSG_SOUND_REGISTERS )
 	{
 		/* Copy sound related registers 0..13 to the sound module's internal buffer */
