@@ -20,12 +20,26 @@ const char ScanDir_fileid[] = "Hatari scandir.c : " __DATE__ " " __TIME__;
 #include "log.h"
 
 /*-----------------------------------------------------------------------
- * Here come alphasort and scandir for BeOS and SunOS
+ * Here come alphasort and scandir for POSIX-like OSes
  *-----------------------------------------------------------------------*/
-#if defined(__BEOS__) || (defined(__sun) && defined(__SVR4))
+#if !defined(WIN32) && !defined(__CEGCC__)
+
+
+/**
+ * Alphabetic order comparison routine.
+ */
+#if !HAVE_ALPHASORT
+int alphasort(const void *d1, const void *d2)
+{
+	return strcmp((*(struct dirent * const *)d1)->d_name,
+		      (*(struct dirent * const *)d2)->d_name);
+}
+#endif
+
+
+#if !HAVE_SCANDIR
 
 #undef DIRSIZ
-
 #define DIRSIZ(dp)                                          \
 		((sizeof(struct dirent) - sizeof(dp)->d_name) +     \
 		(((dp)->d_reclen + 1 + 3) &~ 3))
@@ -37,17 +51,6 @@ const char ScanDir_fileid[] = "Hatari scandir.c : " __DATE__ " " __TIME__;
 #endif
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * Alphabetic order comparison routine.
- */
-int alphasort(const void *d1, const void *d2)
-{
-	return strcmp((*(struct dirent * const *)d1)->d_name, (*(struct dirent * const *)d2)->d_name);
-}
-
-
-/*-----------------------------------------------------------------------*/
 /**
  * Scan a directory for all its entries
  * Return -1 on error, number of entries on success
@@ -140,9 +143,10 @@ error_out:
 		closedir(dirp);
 	return -1;
 }
+#endif	/* !HAVE_SCANDIR */
 
 
-#endif /* __BEOS__ || __sun */
+#endif /* !WIN32 */
 
 
 /*-----------------------------------------------------------------------
