@@ -66,6 +66,7 @@
 /* 2008/10/26	[NP]	Correctly save/restore all necessary variables in		*/
 /*			Sound_MemorySnapShot_Capture.					*/
 /* 2008/11/23	[NP]	Clean source, remove old sound core.				*/
+/* 2011/11/03	[DS]	Stereo DC filtering which accounts for DMA sound.               */
 
 
 
@@ -146,16 +147,16 @@ static const ymu16 ymout1c5bit[ 32 ] =
 };
 
 /* Convert a constant 4 bits volume to the internal 5 bits value : */
-/* volume5=volume4*2+1, except for volumes 0 and 1 which become 0 and 2, */
+/* volume5=volume4*2+1, except for volumes 0 and 1 which remain 0 and 1, */
 /* in order to map [0,15] into [0,31] (O must remain 0, and 15 must give 31) */
-static const ymu16 YmVolume4to5[ 16 ] = { 0,2,5,7,9,11,13,15,17,19,21,23,25,27,29,31 };
+static const ymu16 YmVolume4to5[ 16 ] = { 0,1,5,7,9,11,13,15,17,19,21,23,25,27,29,31 };
 
 /* Table of unsigned 4 bit D/A output level for 3 channels as measured on a real ST */
-static ymu16 volumetable_original[ 16 * 16 * 16 ] =
+static ymu16 volumetable_original[16][16][16] =
 #include "ym2149_fixed_vol.h"
 
 /* Corresponding table interpolated to 5 bit D/A output level (16 bits unsigned) */
-static ymu16 ymout5_u16[ 32 * 32 * 32 ];
+static ymu16 ymout5_u16[32][32][32];
 
 /* Same table, after conversion to signed results (same pointer, with different type) */
 static yms16 *ymout5 = (yms16 *)ymout5_u16;
@@ -190,18 +191,6 @@ static yms16 *ymout5 = (yms16 *)ymout5_u16;
 
 
 
-/*--------------------------------------------------------------*/
-/* Variables for the DC adjuster / Low Pass Filter		*/
-/*--------------------------------------------------------------*/
-#define DC_ADJUST_BUFFERLEN		512		/* must be a power of 2 */
-
-static ymsample	dc_buffer[DC_ADJUST_BUFFERLEN];
-static int	dc_pos;
-static int	dc_sum;
-static ymsample	m_lowPassFilter[2];
-
-
-
 /*--------------------------------------------------------------*/
 /* Variables for the YM2149 emulator (need to be saved and	*/
 /* restored in memory snapshots)				*/
@@ -250,18 +239,13 @@ bool		Sound_BufferIndexNeedReset = false;
 /* Local functions prototypes					*/
 /*--------------------------------------------------------------*/
 
-static void	DcAdjuster_Reset	(void);
-static void	DcAdjuster_AddSample	(ymsample sample);
-static ymsample	DcAdjuster_GetDcLevel	(void);
-static void	LowPassFilter_Reset	(void);
-static ymsample	LowPassFilter		(ymsample in);
+static ymsample	LowPassFilter		(ymsample x0);
+static ymsample	PWMaliasFilter		(ymsample x0);
 
-static int	volumetable_get		(int i, int j, int k);
-static void	volumetable_set		(ymu16 *volumetable, int i, int j, int k, int val);
-static int	volumetable_interpolate	(int y1, int y2);
-static void	interpolate_volumetable	(ymu16 *out);
+static void	interpolate_volumetable	(ymu16 volumetable[32][32][32]);
 
-static void	YM2149_BuildLinearVolumeTable(ymu16 *out);
+static void	YM2149_BuildModelVolumeTable(ymu16 volumetable[32][32][32]);
+static void	YM2149_BuildLinearVolumeTable(ymu16 volumetable[32][32][32]);
 static void	YM2149_Normalise_5bit_Table(ymu16 *in_5bit , yms16 *out_5bit, unsigned int Level, bool DoCenter);
 
 static void	YM2149_EnvBuild		(void);
@@ -281,52 +265,119 @@ static void	Sound_GenerateSamples(int SamplesToGenerate);
 
 
 /*--------------------------------------------------------------*/
-/* DC Adjuster / Low Pass Filter routines.			*/
+/* DC Adjuster							*/
 /*--------------------------------------------------------------*/
 
-static void	DcAdjuster_Reset(void)
+/**
+ * 6dB/octave first order HPF fc = (1.0-0.998)*44100/(2.0*pi)
+ * Z pole = 0.99804 --> FS = 44100 Hz : fc=13.7 Hz (11 Hz meas)
+ * a = (int32_t)(32768.0*(1.0 - pole)) :       a = 64 !!!
+ * Input range: -32768 to 32767  Maximum step: +65536 or -65472
+ */
+ymsample	Subsonic_IIR_HPF_Left(ymsample x0)
 {
-	int	i;
+	static	yms32	x1 = 0, y1 = 0, y0 = 0;
 
-	for (i=0 ; i<DC_ADJUST_BUFFERLEN ; i++)
-		dc_buffer[i] = 0;
+	y1 += ((x0 - x1)<<15) - (y0<<6);  /*  64*y0  */
+	y0 = y1>>15;
+	x1 = x0;
 
-	dc_pos = 0;
-	dc_sum = 0;
+	return y0;
 }
 
 
-static void	DcAdjuster_AddSample(ymsample sample)
+ymsample	Subsonic_IIR_HPF_Right(ymsample x0)
 {
-	dc_sum -= dc_buffer[dc_pos];
-	dc_sum += sample;
+	static	yms32	x1 = 0, y1 = 0, y0 = 0;
 
-	dc_buffer[dc_pos] = sample;
-	dc_pos = (dc_pos+1)&(DC_ADJUST_BUFFERLEN-1);
+	y1 += ((x0 - x1)<<15) - (y0<<6);  /*  64*y0  */
+	y0 = y1>>15;
+	x1 = x0;
+
+	return y0;
 }
 
 
-static ymsample	DcAdjuster_GetDcLevel(void)
+/*--------------------------------------------------------------*/
+/* Low Pass Filter routines.					*/
+/*--------------------------------------------------------------*/
+
+/**
+ * Get coefficients for different Fs (C10 is in ST only):
+ * Wc = 2*M_PI*4895.1;
+ * Fs = 44100;
+ * warp = Wc/tanf((Wc/2)/Fs);
+ * b = Wc/(warp+Wc);
+ * a = (Wc-warp)/(warp+Wc);
+ *
+ * #define B_z (yms32)( 0.2667*(1<<15))
+ * #define A_z (yms32)(-0.4667*(1<<15))
+ *
+ * y0 = (B_z*(x0 + x1) - A_z*y0) >> 15;
+ * x1 = x0;
+ *
+ * The Lowpass Filter formed by C10 = 0.1 uF
+ * and
+ * R8=1k // 1k*(65119-46602)/65119 // R9=10k // R10=5.1k //
+ * (R12=470)*(100=Q1_HFE) = 206.865 ohms when YM2149 is High
+ * and
+ * R8=1k // R9=10k // R10=5.1k // (R12=470)*(100=Q1_HFE)
+ *                        = 759.1   ohms when YM2149 is Low
+ * High corner is 1/(2*pi*(0.1*10e-6)*206.865) fc = 7693.7 Hz
+ * Low  corner is 1/(2*pi*(0.1*10e-6)*795.1)   fc = 2096.6 Hz
+ * Notes:
+ * - using STF reference designators R8 R9 R10 C10 (from dec 1986 schematics)
+ * - using corresponding numbers from psgstrep and psgquart
+ * - 65119 is the largest value in Paulo's psgstrep table
+ * - 46602 is the largest value in Paulo's psgquart table
+ * - this low pass filter uses the highest cutoff frequency
+ *   on the STf (a slightly lower frequency is reasonable).
+ *
+ * A first order lowpass filter with a high cutoff frequency
+ * is used when the YM2149 pulls high, and a lowpass filter
+ * with a low cutoff frequency is used when R8 pulls low.
+ */
+static ymsample	LowPassFilter(ymsample x0)
 {
-	return dc_sum / DC_ADJUST_BUFFERLEN;
+	static	yms32 y0 = 0, x1 = 0;
+
+	if (x0 >= y0)
+	/* YM Pull up:   fc = 7586.1 Hz (44.1 KHz), fc = 8257.0 Hz (48 KHz) */
+		y0 = (3*(x0 + x1) + (y0<<1)) >> 3;
+	else
+	/* R8 Pull down: fc = 1992.0 Hz (44.1 KHz), fc = 2168.0 Hz (48 KHz) */
+		y0 = ((x0 + x1) + (6*y0)) >> 3;
+
+	x1 = x0;
+	return y0;
 }
 
-
-static void	LowPassFilter_Reset(void)
+/**
+ * This piecewise selective filter works by filtering the falling
+ * edge of a sampled pulse-wave differently from the rising edge.
+ *
+ * Piecewise selective filtering is effective because harmonics on
+ * one part of a wave partially define harmonics on other portions.
+ *
+ * Piecewise selective filtering can efficiently reduce aliasing
+ * with minimal harmonic removal.
+ *
+ * I disclose this information into the public domain so that it
+ * cannot be patented. May 23 2012 David Savinkoff.
+ */
+static ymsample	PWMaliasFilter(ymsample x0)
 {
-	m_lowPassFilter[0] = 0;
-	m_lowPassFilter[1] = 0;
-}
+	static	yms32 y0 = 0, x1 = 0;
 
+	if (x0 >= y0)
+	/* YM Pull up   */
+		y0 = x0;
+	else
+	/* R8 Pull down */
+		y0 = (3*(x0 + x1) + (y0<<1)) >> 3;
 
-static ymsample	LowPassFilter(ymsample in)
-{
-	ymsample	out;
- 
-	out = (m_lowPassFilter[0]>>2) + (m_lowPassFilter[1]>>1) + (in>>2);
-	m_lowPassFilter[0] = m_lowPassFilter[1];
-	m_lowPassFilter[1] = in;
-	return out;
+	x1 = x0;
+	return y0;
 }
 
 
@@ -338,108 +389,43 @@ static ymsample	LowPassFilter(ymsample in)
 /* of all possible fixed volume combinations on a ST.		*/
 /*--------------------------------------------------------------*/
 
-static int	volumetable_get(int i, int j, int k)
+static void	interpolate_volumetable(ymu16 volumetable[32][32][32])
 {
-	/* access at boundary finds the last value instead of the first */
-	if (i == 16)
-		i = 15;
-	if (j == 16)
-		j = 15;
-	if (k == 16)
-		k = 15;
+	int	i, j, k;
 
-	return volumetable_original[i + 16 * j + 16 * 16 * k];
-}
-
-static void	volumetable_set(ymu16 *volumetable, int i, int j, int k, int val)
-{
-	volumetable[i + 32 * j + 32 * 32 * k] = val;
-}
-
-/* the table is exponential in nature. These weighing factors approximate
- * that the value in-between needs to be closer to the lower value in y2 */
-static int	volumetable_interpolate(int y1, int y2)
-{
-	int erpolate;
-	erpolate = (y1 * 4 + y2 * 6) / 10u;
-	if (erpolate > 65535)
-	{
-//		fprintf ( stderr , "sature>:%d %d %d\n",erpolate,y1,y2 );
-		erpolate = 65535;
-	}
-	if (erpolate < 0)
-	{
-//		fprintf ( stderr , "sature<:%d %d %d\n",erpolate,y1,y2 );
-		erpolate = 0;
-	}
-	return erpolate;
-}
-
-static void	interpolate_volumetable(ymu16 *out)
-{
-	int i, j, k;
-	int i1, i2;
-
-	/* we are doing 4-dimensional interpolation here. For each
-	* known measurement point, we must find 8 new values. These values occur
-	* as follows:
-	*
-	* - one at the exact same position
-	* - one half-way in i direction
-	* - one half-way in j direction
-	* - one half-way in k direction
-	* - one half-way in i+j direction
-	* - one half-way in i+k direction
-	* - one half-way in j+k direction
-	* - one half-way in i+j+k direction
-	*
-	* The algorithm currently is very simplistic. Probably more points should be
-	* weighted in the multicomponented directions, for instance i+j+k should be
-	* an average of all surrounding data points. This probably doesn't matter much,
-	* though. This is because the only way to reach those locations is to modulate
-	* more than one voice with the envelope, and this is rare.
-	*/
-
-	for (i = 0; i < 16; i++)
-	{
-		for (j = 0; j < 16; j++)
-		{
-			for (k = 0; k < 16; k++)
-			{
-				i1 = volumetable_get(i, j, k);
-				/* copy value unchanged to new position */
-				volumetable_set(out,i*2, j*2, k*2, i1);
- 
-				/* interpolate in i direction */
-				i2 = volumetable_get(i + 1, j, k);
-				volumetable_set(out,i*2 + 1, j*2, k*2, volumetable_interpolate(i1, i2));
-
-				/* interpolate in j direction */
-				i2 = volumetable_get(i, j+1, k);
-				volumetable_set(out,i*2, j*2 + 1, k*2, volumetable_interpolate(i1, i2));
-
-				/* interpolate in k direction */
-				i2 = volumetable_get(i, j, k+1);
-				volumetable_set(out,i*2, j*2, k*2+1, volumetable_interpolate(i1, i2));
-
-				/* interpolate in i + j direction */
-				i2 = volumetable_get(i + 1, j + 1, k);
-				volumetable_set(out,i*2 + 1, j*2 + 1, k*2, volumetable_interpolate(i1, i2));
-
-				/* interpolate in i + k direction */
-				i2 = volumetable_get(i + 1, j, k + 1);
-				volumetable_set(out,i*2 + 1, j*2, k*2 + 1, volumetable_interpolate(i1, i2));
-
-				/* interpolate in j + k direction */
-				i2 = volumetable_get(i, j + 1, k + 1);
-				volumetable_set(out,i*2, j*2 + 1, k*2 + 1, volumetable_interpolate(i1, i2));
-
-				/* interpolate in i + j + k direction */
-				i2 = volumetable_get(i + 1, j + 1, k + 1);
-				volumetable_set(out,i*2 + 1, j*2 + 1, k*2 + 1, volumetable_interpolate(i1, i2));
+	for (i = 1; i < 32; i += 2) { /* Copy 16 Panels to make a Block */
+		for (j = 1; j < 32; j += 2) { /* Copy 16 Rows to make a Panel */
+			for (k = 1; k < 32; k += 2) { /* Copy 16 Elements to make a Row */
+				volumetable[i][j][k] = volumetable_original[(i-1)/2][(j-1)/2][(k-1)/2];
 			}
+			volumetable[i][j][0] = volumetable[i][j][1]; /* Move 0th Element */
+			volumetable[i][j][1] = volumetable[i][j][3]; /* Move 1st Element */
+			/* Interpolate 3rd Element */
+			volumetable[i][j][3] = (ymu16)(0.5 + sqrt((double)volumetable[i][j][1] * volumetable[i][j][5]));
+			for (k = 2; k < 32; k += 2) /* Interpolate Even Elements */
+				volumetable[i][j][k] = (ymu16)(0.5 + sqrt((double)volumetable[i][j][k-1] * volumetable[i][j][k+1]));
 		}
+		for (k = 0; k < 32; k++) {
+			volumetable[i][0][k] = volumetable[i][1][k]; /* Move 0th Row */
+			volumetable[i][1][k] = volumetable[i][3][k]; /* Move 1st Row */
+			/* Interpolate 3rd Row */
+			volumetable[i][3][k] = (ymu16)(0.5 + sqrt((double)volumetable[i][1][k] * volumetable[i][5][k]));
+		}
+		for (j = 2; j < 32; j += 2) /* Interpolate Even Rows */
+			for (k = 0; k < 32; k++)
+				volumetable[i][j][k] = (ymu16)(0.5 + sqrt((double)volumetable[i][j-1][k] * volumetable[i][j+1][k]));
 	}
+	for (j = 0; j < 32; j++)
+		for (k = 0; k < 32; k++) {
+			volumetable[0][j][k] = volumetable[1][j][k]; /* Move 0th Panel */
+			volumetable[1][j][k] = volumetable[3][j][k]; /* Move 1st Panel */
+			/* Interpolate 3rd Panel */
+			volumetable[3][j][k] = (ymu16)(0.5 + sqrt((double)volumetable[1][j][k] * volumetable[5][j][k]));
+		}
+	for (i = 2; i < 32; i += 2) /* Interpolate Even Panels */
+		for (j = 0; j < 32; j++) /* Interpolate Even Panels */
+			for (k = 0; k < 32; k++)
+				volumetable[i][j][k] = (ymu16)(0.5 + sqrt((double)volumetable[i-1][j][k] * volumetable[i+1][j][k]));
 }
 
 
@@ -452,18 +438,132 @@ static void	interpolate_volumetable(ymu16 *out)
  * (each value of ymout1c5bit is in [0,65535])
  */
 
-static void	YM2149_BuildLinearVolumeTable(ymu16 *out)
+static void	YM2149_BuildLinearVolumeTable(ymu16 volumetable[32][32][32])
 {
 	int	i, j, k;
-	int	res;
 
+	for (i = 0; i < 32; i++)
+		for (j = 0; j < 32; j++)
+			for (k = 0; k < 32; k++)
+				volumetable[i][j][k] = (ymu16)( ((ymu32)ymout1c5bit[i] + ymout1c5bit[j] + ymout1c5bit[k]) / 3);
+}
+
+
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Build a circuit analysed version of the conversion table.
+ * David Savinkoff designed this algorithm by analysing data
+ * measured by Paulo Simoes and Benjamin Gerard.
+ * The numbers are arrived at by assuming a current steering
+ * resistor ladder network and using the voltage divider rule.
+ *
+ * If one looks at the ST schematic of the YM2149, one sees
+ * three sound pins tied together and attached to a 1000 ohm
+ * resistor (1k) that has the other end grounded.
+ * The 1k resistor is also in parallel with a 0.1 microfarad
+ * capacitor (on the Atari ST, not STE or others). The voltage
+ * developed across the 1K resistor is the output voltage which
+ * I call Vout.
+ *
+ * The output of the YM2149 is modeled well as pullup resistors.
+ * Thus, the three sound pins are seen as three parallel
+ * computer-controlled, adjustable pull-up resistors.
+ * To emulate the output of the YM2149, one must determine the
+ * resistance values of the YM2149 relative to the 1k resistor,
+ * which is done by the 'math model'.
+ *
+ * The AC + DC math model is:
+ *
+ * (MaxVol*WARP) / (1.0 + 1.0/(conductance_[i]+conductance_[j]+conductance_[k]))
+ * or
+ * (MaxVol*WARP) / (1.0 + 1.0/( 1/Ra +1/Rb  +1/Rc )) , Ra = channel A resistance
+ *
+ * Note that the first 1.0 in the formula represents the
+ * normalized 1k resistor (1.0 * 1000 ohms = 1k).
+ *
+ * The YM2149 DC component model represents the output voltage
+ * filtered of high frequency AC component, but DC component
+ * remains.
+ * The YM2149 DC component mode treats the voltage exactly as if
+ * it were low pass filtered. This is more than what is required
+ * to make 'quartet mode sound'. Simplicity leads to Generality!
+ *
+ * The DC component model model is:
+ *
+ * (MaxVol*WARP) / (2.0 + 1.0/( 1/Ra + 1/Rb  + 1/Rc))
+ * or
+ * (MaxVol*WARP*0.5) / (1.0 + 0.5/( 1/Ra + 1/Rb  + 1/Rc))
+ *
+ * Note that the 1.0 represents the normalized 1k resistor.
+ * 0.5 represents 50% duty cycle for the parallel resistors
+ * being summed (this effectively doubles the pull-up resistance).
+ */
+
+static void	YM2149_BuildModelVolumeTable(ymu16 volumetable[32][32][32])
+{
+#define MaxVol  65535.0               /* Normal Mode Maximum value in table */
+#define FOURTH2 1.19                  /* Fourth root of two from YM2149 */
+#define WARP    1.666666666666666667  /* measured as 1.65932 from 46602 */
+
+	double conductance;
+	double conductance_[32];
+	int	i, j, k;
+
+/**
+ * YM2149 and R8=1k follows (2^-1/4)^(n-31) better when 2 voices are
+ * summed (A+B or B+C or C+A) rather than individually (A or B or C):
+ *   conductance = 2.0/3.0/(1.0-1.0/WARP)-2.0/3.0;
+ * When taken into consideration with three voices.
+ *
+ * Note that the YM2149 does not use laser trimmed resistances, thus
+ * has offsets that are added and/or multiplied with (2^-1/4)^(n-31).
+ */
+	conductance = 2.0/3.0/(1.0-1.0/WARP)-2.0/3.0; /* conductance = 1.0 */
+
+/**
+ * Because the YM current output (voltage source with series resistances)
+ * is connected to a grounded resistor to develop the output voltage
+ * (instead of a current to voltage converter), the output transfer
+ * function is not linear. Thus:
+ * 2.0*conductance_[n] = 1.0/(1.0-1.0/FOURTH2/(1.0/conductance + 1.0))-1.0;
+ */
+	for (i = 31; i >= 1; i--)
+	{
+		conductance_[i] = conductance/2.0;
+		conductance = 1.0/(1.0-1.0/FOURTH2/(1.0/conductance + 1.0))-1.0;
+	}
+	conductance_[0] = 1.0e-8; /* Avoid divide by zero */
+
+/**
+ * YM2149 AC + DC components model:
+ * (Note that Maxvol is 65119 in Simoes' table, 65535 in Gerard's)
+ *
+ * Sum the conductances as a function of a voltage divider:
+ * Vout=Vin*Rout/(Rout+Rin)
+ */
 	for (i = 0; i < 32; i++)
 		for (j = 0; j < 32; j++)
 			for (k = 0; k < 32; k++)
 			{
-				res = ( ymout1c5bit[ i ] + ymout1c5bit[ j ] + ymout1c5bit[ k ] ) / 3;
-				volumetable_set ( out, i, j, k, res );
+				volumetable[i][j][k] = (ymu16)(0.5+(MaxVol*WARP)/(1.0 +
+					1.0/(conductance_[i]+conductance_[j]+conductance_[k])));
 			}
+
+/**
+ * YM2149 DC component model:
+ * R8=1k (pulldown) + YM//1K (pullup) with YM 50% duty PWM
+ * (Note that MaxVol is 46602 in Paulo Simoes Quartet mode table)
+ *
+ *	for (i = 0; i < 32; i++)
+ *		for (j = 0; j < 32; j++)
+ *			for (k = 0; k < 32; k++)
+ *			{
+ *				volumetable[i][j][k] = (ymu16)(0.5+(MaxVol*WARP)/(1.0 +
+ *					2.0/(conductance_[i]+conductance_[j]+conductance_[k])));
+ *			}
+ */
 }
 
 
@@ -557,13 +657,15 @@ static void	Ym2149_BuildVolumeTable(void)
 {
 	/* Depending on the volume mixing method, we use a table based on real measures */
 	/* or a table based on a linear volume mixing. */
-	if ( YmVolumeMixing == YM_TABLE_MIXING )
+	if ( YmVolumeMixing == YM_MODEL_MIXING )
+		YM2149_BuildModelVolumeTable(ymout5_u16);	/* create 32*32*32 circuit analysed model of the volume table */
+	else if ( YmVolumeMixing == YM_TABLE_MIXING )
 		interpolate_volumetable(ymout5_u16);		/* expand the 16*16*16 values in volumetable_original to 32*32*32 */
 	else
 		YM2149_BuildLinearVolumeTable(ymout5_u16);	/* combine the 32 possible volumes */
 
 	/* Normalise/center the values (convert from u16 to s16) */
-	YM2149_Normalise_5bit_Table ( ymout5_u16 , ymout5 , YM_OUTPUT_LEVEL , YM_OUTPUT_CENTERED );
+	YM2149_Normalise_5bit_Table ( ymout5_u16[0][0] , ymout5 , YM_OUTPUT_LEVEL , YM_OUTPUT_CENTERED );
 }
 
 
@@ -612,9 +714,6 @@ static void	Ym2149_Reset(void)
 
 	envShape = 0;
 	envPos = 0;
-
-	DcAdjuster_Reset ();
-	LowPassFilter_Reset ();
 }
 
 
@@ -657,7 +756,8 @@ static ymu32	Ym2149_ToneStepCompute(ymu8 rHigh , ymu8 rLow)
 
 	per = rHigh&15;
 	per = (per<<8)+rLow;
-	if (per <= 5)
+
+	if  (per <= (int)(YM_ATARI_CLOCK/(YM_REPLAY_FREQ*7)) )
 		return 0;
 
 	step = YM_ATARI_CLOCK;
@@ -679,8 +779,8 @@ static ymu32	Ym2149_ToneStepCompute(ymu8 rHigh , ymu8 rLow)
 	if ( per == 0 )
 		per = 1;				/* result for Per=0 is the same as for Per=1 */	
 #else
-	if ( per <= 5 )
-		return 0;				/* discard too high frequencies, they give a very bad sound */	
+	if  (per <= (int)(YM_ATARI_CLOCK/(YM_REPLAY_FREQ*7)) )
+		return 0;				/* discard frequencies higher than 80% of nyquist rate. */
 #endif
 
 	step = YM_ATARI_CLOCK;
@@ -830,7 +930,21 @@ static ymsample	YM2149_NextSample(void)
 	/* volumes depending on the output state of each voice (0 or 0x1f) */
 	Tone3Voices &= ( Env3Voices | Vol3Voices );
 
+	/* When a step period is 0, the represented frequency was filtered from the */
+	/* ouput of the YM2149. Thus, use the transient DC component of the sample. */
+	/* Note that the "-1" table offset is a "good fit" for the DC component.    */
+
+	if (stepA == 0  &&  (Tone3Voices & YM_MASK_A) > 1)
+		Tone3Voices -= 1;     /* Voice A AC component removed; Transient DC component remains */
+
+	if (stepB == 0  &&  (Tone3Voices & YM_MASK_B) > 1<<5)
+		Tone3Voices -= 1<<5;  /* Voice B AC component removed; Transient DC component remains */
+
+	if (stepC == 0  &&  (Tone3Voices & YM_MASK_C) > 1<<10)
+		Tone3Voices -= 1<<10; /* Voice C AC component removed; Transient DC component remains */
+
 	/* D/A conversion of the 3 volumes into a sample using a precomputed conversion table */
+
 	sample = ymout5[ Tone3Voices ];			/* 16 bits signed value */
 
 
@@ -844,16 +958,11 @@ static ymsample	YM2149_NextSample(void)
 	if ( envPos >= (3*32) << 24 )			/* blocks 0, 1 and 2 were used (envPos 0 to 95) */
 		envPos -= (2*32) << 24;			/* replay/loop blocks 1 and 2 (envPos 32 to 95) */
 
-	DcAdjuster_AddSample(sample);			/* Calculate DC level */
-	sample = sample - DcAdjuster_GetDcLevel();	/* normalize sound level */
-
 	/* Apply low pass filter ? */
 	if ( UseLowPassFilter )
-	{
-		sample = LowPassFilter(sample);
-	}
-
-	return sample;
+		return LowPassFilter(sample);
+	else
+		return PWMaliasFilter(sample);
 }
 #else
 static ymsample	YM2149_NextSample(void)
@@ -895,6 +1004,16 @@ static ymsample	YM2149_NextSample(void)
 	Tone3Voices &= ( Env3Voices | Vol3Voices );
 
 	/* D/A conversion of the 3 volumes into a sample using a precomputed conversion table */
+
+	if (stepA == 0  &&  (Tone3Voices & YM_MASK_A) > 1)
+		Tone3Voices -= 1;     /* Voice A AC component removed; Transient DC component remains */
+
+	if (stepB == 0  &&  (Tone3Voices & YM_MASK_B) > 1<<5)
+		Tone3Voices -= 1<<5;  /* Voice B AC component removed; Transient DC component remains */
+
+	if (stepC == 0  &&  (Tone3Voices & YM_MASK_C) > 1<<10)
+		Tone3Voices -= 1<<10; /* Voice C AC component removed; Transient DC component remains */
+
 	sample = ymout5[ Tone3Voices ];			/* 16 bits signed value */
 
 
@@ -908,16 +1027,11 @@ static ymsample	YM2149_NextSample(void)
 	if ( envPos >= (3*32) << 24 )			/* blocks 0, 1 and 2 were used (envPos 0 to 95) */
 		envPos -= (2*32) << 24;			/* replay/loop blocks 1 and 2 (envPos 32 to 95) */
 
-	DcAdjuster_AddSample(sample);			/* Calculate DC level */
-	sample = sample - DcAdjuster_GetDcLevel();	/* normalize sound level */
-
 	/* Apply low pass filter ? */
 	if ( UseLowPassFilter )
-	{
-		sample = LowPassFilter(sample);
-	}
-
-	return sample;
+		return LowPassFilter(sample);
+	else
+		return PWMaliasFilter(sample);
 }
 #endif
 
@@ -1250,7 +1364,7 @@ static void Sound_GenerateSamples(int SamplesToGenerate)
 		for (i = 0; i < SamplesToGenerate; i++)
 		{
 			idx = (ActiveSndBufIdx + i) % MIXBUFFER_SIZE;
-			MixBuffer[idx][0] = MixBuffer[idx][1] = YM2149_NextSample();
+			MixBuffer[idx][0] = MixBuffer[idx][1] = Subsonic_IIR_HPF_Left( YM2149_NextSample() );
 		}
  		/* If Falcon emulation, crossbar does the job */
  		Crossbar_GenerateSamples(ActiveSndBufIdx, SamplesToGenerate);
@@ -1260,9 +1374,9 @@ static void Sound_GenerateSamples(int SamplesToGenerate)
 		for (i = 0; i < SamplesToGenerate; i++)
 		{
 			idx = (ActiveSndBufIdx + i) % MIXBUFFER_SIZE;
-			MixBuffer[idx][0] = MixBuffer[idx][1] = (YM2149_NextSample() >> 1);
+			MixBuffer[idx][0] = MixBuffer[idx][1] = YM2149_NextSample();
 		}
- 		/* If Ste or TT emulation, DmaSnd does the job */
+ 		/* If Ste or TT emulation, DmaSnd does mixing and filtering */
  		DmaSnd_GenerateSamples(ActiveSndBufIdx, SamplesToGenerate);
 	}
 	else if (ConfigureParams.System.nMachineType == MACHINE_ST)
@@ -1270,10 +1384,8 @@ static void Sound_GenerateSamples(int SamplesToGenerate)
 		for (i = 0; i < SamplesToGenerate; i++)
 		{
 			idx = (ActiveSndBufIdx + i) % MIXBUFFER_SIZE;
-			MixBuffer[idx][0] = MixBuffer[idx][1] = YM2149_NextSample();
+			MixBuffer[idx][0] = MixBuffer[idx][1] = Subsonic_IIR_HPF_Left( YM2149_NextSample() );
 		}
-		/* If ST emulation, DmaSnd doesn't do the job
-		DmaSnd_GenerateSamples(ActiveSndBufIdx, nSamplesToGenerate); */
  	}
 
 	ActiveSndBufIdx = (ActiveSndBufIdx + SamplesToGenerate) % MIXBUFFER_SIZE;
