@@ -47,7 +47,8 @@ const char Statusbar_fileid[] = "Hatari statusbar.c : " __DATE__ " " __TIME__;
 
 #define DEBUG 0
 #if DEBUG
-#define DEBUGPRINT(x) printf x
+#include <execinfo.h>
+# define DEBUGPRINT(x) printf x
 #else
 #define DEBUGPRINT(x)
 #endif
@@ -85,7 +86,7 @@ static Uint32 LedColorOn, LedColorOff, RecColorOn, RecColorOff;
 static Uint32 GrayBg, LedColorBg;
 
 
-#define MAX_MESSAGE_LEN 33
+#define MAX_MESSAGE_LEN 35
 typedef struct msg_item {
 	struct msg_item *next;
 	char msg[MAX_MESSAGE_LEN+1];
@@ -120,11 +121,14 @@ int Statusbar_GetHeightForSize(int width, int height)
 		 * does to decide the font size.
 		 */
 		if (width >= 640 && height >= (400-24)) {
+			DEBUGPRINT(("Statusbar_GetHeightForSize(%d, %d) -> %d\n", width, height, 24));
 			return 24;
 		} else {
+			DEBUGPRINT(("Statusbar_GetHeightForSize(%d, %d) -> %d\n", width, height, 12));
 			return 12;
 		}
 	}
+	DEBUGPRINT(("Statusbar_GetHeightForSize(%d, %d) -> %d\n", width, height, 0));
 	return 0;
 }
 
@@ -138,8 +142,15 @@ int Statusbar_GetHeightForSize(int width, int height)
  */
 int Statusbar_SetHeight(int width, int height)
 {
+#if DEBUG
+	/* find out from where the set height is called */
+	void *addr[8];
+	int count = backtrace(addr, sizeof(addr)/sizeof(*addr));
+	backtrace_symbols_fd(addr, count, fileno(stderr));
+#endif
 	ScreenHeight = height;
 	StatusbarHeight = Statusbar_GetHeightForSize(width, height);
+	DEBUGPRINT(("Statusbar_SetHeight(%d, %d) -> %d\n", width, height, StatusbarHeight));
 	return StatusbarHeight;
 }
 
@@ -214,6 +225,7 @@ void Statusbar_Init(SDL_Surface *surf)
 	int i, fontw, fonth, offset;
 	const char *text[MAX_DRIVE_LEDS] = { "A:", "B:", "HD:" };
 
+	DEBUGPRINT(("Statusbar_Init()\n"));
 	assert(surf);
 
 	/* dark green and light green for leds themselves */
@@ -236,6 +248,7 @@ void Statusbar_Init(SDL_Surface *surf)
 		StatusbarHeight = 0;
 	}
 	if (!StatusbarHeight) {
+		DEBUGPRINT(("Doesn't fit <- Statusbar_Init()\n"));
 		return;
 	}
 
@@ -324,7 +337,7 @@ void Statusbar_Init(SDL_Surface *surf)
 
 	/* and blit statusbar on screen */
 	SDL_UpdateRects(surf, 1, &sbarbox);
-	DEBUGPRINT(("Draw statusbar\n"));
+	DEBUGPRINT(("Drawn <- Statusbar_Init()\n"));
 }
 
 
@@ -402,7 +415,7 @@ void Statusbar_UpdateInfo(void)
 		if (ConfigureParams.Memory.nMemorySize) {
 			*end++ = '0' + ConfigureParams.Memory.nMemorySize;
 		} else {
-			end = Statusbar_AddString(end, "1/2");
+			end = Statusbar_AddString(end, "0.5");
 		}
 	}
 	end = Statusbar_AddString(end, "MB ");
@@ -416,7 +429,7 @@ void Statusbar_UpdateInfo(void)
 		end = Statusbar_AddString(end, "STE");
 		break;
 	case MACHINE_MEGA_STE:
-		end = Statusbar_AddString(end, "Mega STE");
+		end = Statusbar_AddString(end, "MegaSTE");
 		break;
 	case MACHINE_TT:
 		end = Statusbar_AddString(end, "TT");
@@ -629,6 +642,11 @@ void Statusbar_Update(SDL_Surface *surf)
 	}
 	assert(surf);
 	/* Statusbar_Init() not called before this? */
+#if DEBUG
+	if (surf->h != ScreenHeight + StatusbarHeight) {
+		printf("%d != %d + %d\n", surf->h, ScreenHeight, StatusbarHeight);
+	}
+#endif
 	assert(surf->h == ScreenHeight + StatusbarHeight);
 
 	rect = LedRect;
