@@ -63,6 +63,10 @@
 /*			Use BusCyclePenalty to properly handle the 2/4 cycles added in that case when	*/
 /*			addressing mode	is Ad8r or PC8r	(ULM Demo Menu, Anomaly Demo Intro, DHS		*/
 /*			Sommarhack 2010) (see m68000.h)							*/
+/* 2012/01/29	[NP]	Add refill_prefetch for i_EOR to fix Operation Clean Streets self modified code.*/
+/*			Ugly hack, we need better prefetch emulation (switch to winuae gencpu.c)	*/
+/* 2012/05/05	[NP]	In i_JMP, in case of address error, last_addr_for_exception_3 should not always	*/
+/*			be pc+6, (Sherman Cracktro in No Extra V2 compilation) (e.g. 'jmp (a2)' : pc+2)	*/
 
 
 const char GenCpu_fileid[] = "Hatari gencpu.c : " __DATE__ " " __TIME__;
@@ -968,6 +972,7 @@ static void gen_opcode (unsigned long int opcode)
         genamode (curi->smode, "srcreg", curi->size, "src", 1, 0);
         genamode (curi->dmode, "dstreg", curi->size, "dst", 1, 0);
         printf ("\tsrc %c= dst;\n", curi->mnemo == i_OR ? '|' : curi->mnemo == i_AND ? '&' : '^');
+	printf("\trefill_prefetch (m68k_getpc(), 2);\n");	// FIXME [NP] For Operation Clean Streets - Automation 168, need better prefetch emulation
         genflags (flag_logical, curi->size, "src", "", "");
         genastore ("src", curi->dmode, "dstreg", curi->size, "dst");
         if(curi->size==sz_long && curi->dmode==Dreg)
@@ -1507,7 +1512,7 @@ static void gen_opcode (unsigned long int opcode)
 	genamode (curi->smode, "srcreg", curi->size, "src", 0, 0);
 	if (using_exception_3) {
 	    printf ("\tif (srca & 1) {\n");
-	    printf ("\t\tlast_addr_for_exception_3 = m68k_getpc() + 6;\n");
+	    printf ("\t\tlast_addr_for_exception_3 = m68k_getpc() + %d;\n" , m68k_pc_offset);	// [NP] last_addr is not pc+6
 	    printf ("\t\tlast_fault_for_exception_3 = srca;\n");
 	    printf ("\t\tlast_op_for_exception_3 = opcode; Exception(3,0,M68000_EXC_SRC_CPU); goto %s;\n", endlabelstr);
 	    printf ("\t}\n");
