@@ -160,7 +160,7 @@ static uae_u32 BusErrMem_lget(uaecptr addr)
     if (illegal_mem)
 	write_log ("Bus error lget at %08lx\n", (long)addr);
 
-    M68000_BusError(addr, 1);
+    M68000_BusError(addr, BUS_ERROR_READ);
     return 0;
 }
 
@@ -169,7 +169,7 @@ static uae_u32 BusErrMem_wget(uaecptr addr)
     if (illegal_mem)
 	write_log ("Bus error wget at %08lx\n", (long)addr);
 
-    M68000_BusError(addr, 1);
+    M68000_BusError(addr, BUS_ERROR_READ);
     return 0;
 }
 
@@ -178,7 +178,7 @@ static uae_u32 BusErrMem_bget(uaecptr addr)
     if (illegal_mem)
 	write_log ("Bus error bget at %08lx\n", (long)addr);
 
-    M68000_BusError(addr, 1);
+    M68000_BusError(addr, BUS_ERROR_READ);
     return 0;
 }
 
@@ -187,7 +187,7 @@ static void BusErrMem_lput(uaecptr addr, uae_u32 l)
     if (illegal_mem)
 	write_log ("Bus error lput at %08lx\n", (long)addr);
 
-    M68000_BusError(addr, 0);
+    M68000_BusError(addr, BUS_ERROR_WRITE);
 }
 
 static void BusErrMem_wput(uaecptr addr, uae_u32 w)
@@ -195,7 +195,7 @@ static void BusErrMem_wput(uaecptr addr, uae_u32 w)
     if (illegal_mem)
 	write_log ("Bus error wput at %08lx\n", (long)addr);
 
-    M68000_BusError(addr, 0);
+    M68000_BusError(addr, BUS_ERROR_WRITE);
 }
 
 static void BusErrMem_bput(uaecptr addr, uae_u32 b)
@@ -203,7 +203,7 @@ static void BusErrMem_bput(uaecptr addr, uae_u32 b)
     if (illegal_mem)
 	write_log ("Bus error bput at %08lx\n", (long)addr);
 
-    M68000_BusError(addr, 0);
+    M68000_BusError(addr, BUS_ERROR_WRITE);
 }
 
 static int BusErrMem_check(uaecptr addr, uae_u32 size)
@@ -297,7 +297,7 @@ static uae_u32 SysMem_lget(uaecptr addr)
 {
     if(addr < 0x800 && !regs.s)
     {
-      M68000_BusError(addr, 1);
+      M68000_BusError(addr, BUS_ERROR_READ);
       return 0;
     }
 
@@ -311,7 +311,7 @@ static uae_u32 SysMem_wget(uaecptr addr)
 {
     if(addr < 0x800 && !regs.s)
     {
-      M68000_BusError(addr, 1);
+      M68000_BusError(addr, BUS_ERROR_READ);
       return 0;
     }
 
@@ -325,7 +325,7 @@ static uae_u32 SysMem_bget(uaecptr addr)
 {
     if(addr < 0x800 && !regs.s)
     {
-      M68000_BusError(addr, 1);
+      M68000_BusError(addr, BUS_ERROR_READ);
       return 0;
     }
 
@@ -338,7 +338,7 @@ static void SysMem_lput(uaecptr addr, uae_u32 l)
 {
     if(addr < 0x8 || (addr < 0x800 && !regs.s))
     {
-      M68000_BusError(addr, 0);
+      M68000_BusError(addr, BUS_ERROR_WRITE);
       return;
     }
 
@@ -352,7 +352,7 @@ static void SysMem_wput(uaecptr addr, uae_u32 w)
 {
     if(addr < 0x8 || (addr < 0x800 && !regs.s))
     {
-      M68000_BusError(addr, 0);
+      M68000_BusError(addr, BUS_ERROR_WRITE);
       return;
     }
 
@@ -366,7 +366,7 @@ static void SysMem_bput(uaecptr addr, uae_u32 b)
 {
     if(addr < 0x8 || (addr < 0x800 && !regs.s))
     {
-      M68000_BusError(addr, 0);
+      M68000_BusError(addr, BUS_ERROR_WRITE);
       return;
     }
 
@@ -517,7 +517,7 @@ static void ROMmem_lput(uaecptr addr, uae_u32 b)
     if (illegal_mem)
 	write_log ("Illegal ROMmem lput at %08lx\n", (long)addr);
 
-    M68000_BusError(addr, 0);
+    M68000_BusError(addr, BUS_ERROR_WRITE);
 }
 
 static void ROMmem_wput(uaecptr addr, uae_u32 b)
@@ -525,7 +525,7 @@ static void ROMmem_wput(uaecptr addr, uae_u32 b)
     if (illegal_mem)
 	write_log ("Illegal ROMmem wput at %08lx\n", (long)addr);
 
-    M68000_BusError(addr, 0);
+    M68000_BusError(addr, BUS_ERROR_WRITE);
 }
 
 static void ROMmem_bput(uaecptr addr, uae_u32 b)
@@ -533,7 +533,7 @@ static void ROMmem_bput(uaecptr addr, uae_u32 b)
     if (illegal_mem)
 	write_log ("Illegal ROMmem bput at %08lx\n", (long)addr);
 
-    M68000_BusError(addr, 0);
+    M68000_BusError(addr, BUS_ERROR_WRITE);
 }
 
 static int ROMmem_check(uaecptr addr, uae_u32 size)
